@@ -1,21 +1,20 @@
-/* rt-parser.c
+/*
+ * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * Copyright (C) 2010 Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "rt-parser.h"
@@ -97,7 +96,7 @@ static MarkupTag get_markup_token (const gchar *alltext, const gchar *start, gin
 static MarkupTag get_token (const gchar *alltext, const gchar *start, gint *out_nb_spaces_before, const gchar **out_end,
 			    MarkupTag start_tag);
 
-/**
+/*
  * get_token
  *
  * returns the token type starting from @iter, and positions @out_end to the last used position
@@ -125,7 +124,7 @@ get_token (const gchar *alltext, const gchar *start, gint *out_nb_spaces_before,
 	return MARKUP_NONE;
 }
 
-/**
+/*
  * get_markup_token
  * @alltext: the complete text
  * @start: starting point
@@ -913,7 +912,12 @@ rt_parse_text (const gchar *text)
 			else {
 				gchar *tmp;
 				tmp = remove_newlines_from_base64 (part);
+#if GLIB_CHECK_VERSION(2,20,0)
 				node->binary.data = g_base64_decode_inplace (tmp, (gsize*) & node->binary.binary_length);
+#else
+				node->binary.data = g_base64_decode (tmp, (gsize*) & node->binary.binary_length);
+				g_free (tmp);
+#endif
 			}
 		}
 		else {
