@@ -1,22 +1,21 @@
-/* gdaui-data-cell-renderer-combo.c
- *
- * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
- * Copyright (C) 2003 - 2010 Vivien Malerba <malerba@gnome-db.org>
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
@@ -117,6 +116,7 @@ gdaui_data_cell_renderer_combo_get_type (void)
 			sizeof (GdauiDataCellRendererCombo),
 			0,              /* n_preallocs */
 			(GInstanceInitFunc) gdaui_data_cell_renderer_combo_init,
+			0
 		};
 		
 		cell_text_type =
@@ -349,7 +349,7 @@ gdaui_data_cell_renderer_combo_set_property (GObject *object,
 			GList *gvalues = g_value_get_pointer (value);
 			gchar *str;
 
-			g_assert (g_list_length (gvalues) == datacell->priv->source->shown_n_cols);
+			g_assert (g_list_length (gvalues) == (guint)datacell->priv->source->shown_n_cols);
 			str = render_text_to_display_from_values (gvalues);
 			g_object_set (G_OBJECT (object), "text", str, NULL);
 			g_free (str);
@@ -437,7 +437,7 @@ render_text_to_display_from_values (GList *values)
  * Creates a new #GdauiDataCellRendererCombo which will fill the parameters listed in
  * @source->nodes with values available from @source->data_model.
  * 
- * Return value: (transfer full): the new cell renderer
+ * Returns: (transfer full): the new cell renderer
  **/
 GtkCellRenderer *
 gdaui_data_cell_renderer_combo_new (GdauiSet *paramlist, GdauiSetSource *source)
@@ -569,12 +569,12 @@ static gboolean gdaui_data_cell_renderer_combo_focus_out_event (GtkWidget *widge
 
 static GtkCellEditable *
 gdaui_data_cell_renderer_combo_start_editing (GtkCellRenderer     *cell,
-					      GdkEvent            *event,
-					      GtkWidget           *widget,
+					      G_GNUC_UNUSED GdkEvent            *event,
+					      G_GNUC_UNUSED GtkWidget           *widget,
 					      const gchar         *path,
-					      GdkRectangle        *background_area,
-					      GdkRectangle        *cell_area,
-					      GtkCellRendererState flags)
+					      G_GNUC_UNUSED GdkRectangle        *background_area,
+					      G_GNUC_UNUSED GdkRectangle        *cell_area,
+					      G_GNUC_UNUSED GtkCellRendererState flags)
 {
 	GdauiDataCellRendererCombo *datacell;
 	GtkWidget *combo;
@@ -638,7 +638,7 @@ gdaui_data_cell_renderer_combo_editing_done (GtkCellEditable *combo, GdauiDataCe
 
 
 static gboolean
-gdaui_data_cell_renderer_combo_focus_out_event (GtkWidget *widget, GdkEvent  *event, 
+gdaui_data_cell_renderer_combo_focus_out_event (GtkWidget *widget, G_GNUC_UNUSED GdkEvent  *event,
 						GdauiDataCellRendererCombo *datacell)
 {
   
