@@ -1,21 +1,21 @@
-/* gdaui-entry-wrapper.c
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2003 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gdaui-entry-wrapper.h"
@@ -424,7 +424,6 @@ gdaui_entry_wrapper_set_value_type (GdauiDataEntry *iface, GType type)
 
 	if (mgwrap->priv->type != type) {
 		GValue *value;
-		GdaDataHandler *dh;
 
 		if (mgwrap->priv->value_ref) {
 			gda_value_free (mgwrap->priv->value_ref);
@@ -439,7 +438,6 @@ gdaui_entry_wrapper_set_value_type (GdauiDataEntry *iface, GType type)
 		mgwrap->priv->value_default = gda_value_new_null ();
 
 		/* Set original value */
-		dh = gdaui_data_entry_get_handler (GDAUI_DATA_ENTRY (mgwrap));
 		value = gda_value_new_null ();
 		gdaui_entry_wrapper_set_ref_value (GDAUI_DATA_ENTRY (mgwrap), value);
  		gda_value_free (value);
@@ -476,10 +474,8 @@ gdaui_entry_wrapper_set_value (GdauiDataEntry *iface, const GValue *value)
 			mgwrap->priv->null_forced = TRUE;
 		else
 			mgwrap->priv->null_forced = FALSE;
-		gdaui_entry_shell_set_unknown ((GdauiEntryShell*) iface, FALSE);
 	}
 	else {
-		gdaui_entry_shell_set_unknown ((GdauiEntryShell*) iface, TRUE);
 		(*mgwrap->priv->real_class->real_set_value) (mgwrap, NULL);
 		mgwrap->priv->null_forced = TRUE;
 	}
@@ -495,13 +491,10 @@ gdaui_entry_wrapper_get_value (GdauiDataEntry *iface)
 {
 	GValue *value = NULL;
 	GdauiEntryWrapper *mgwrap;
-	GdaDataHandler *dh;
 
 	g_return_val_if_fail (GDAUI_IS_ENTRY_WRAPPER (iface), NULL);
 	mgwrap = (GdauiEntryWrapper*) iface;
 
-	dh = gdaui_entry_wrapper_get_handler (GDAUI_DATA_ENTRY (mgwrap));
-
 	if (mgwrap->priv->null_forced)
 		value = gda_value_new_null ();
 	else {
@@ -718,8 +711,6 @@ gdaui_entry_wrapper_set_attributes (GdauiDataEntry *iface, GdaValueAttribute att
 		g_warning ("Having an original value is not a write attribute on GdauiDataEntry!");
 
 	current = gdaui_data_entry_get_attributes (iface);
-	gdaui_entry_shell_set_unknown ((GdauiEntryShell*) iface,
-				       current & GDA_VALUE_ATTR_DATA_NON_VALID ? TRUE : FALSE);
 
 	if (signal_contents_changed) {
 		mgwrap->priv->contents_has_changed = FALSE;
