@@ -1,21 +1,22 @@
-/* gdaui-data-entry.c
+/*
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2003 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gdaui-data-entry.h"
@@ -49,7 +50,8 @@ gdaui_data_entry_get_type (void)
 			NULL,
 			0,
 			0,
-			(GInstanceInitFunc) NULL
+			(GInstanceInitFunc) NULL,
+			0
 		};
 
 		type = g_type_register_static (G_TYPE_INTERFACE, "GdauiDataEntry", &info, 0);
@@ -58,10 +60,10 @@ gdaui_data_entry_get_type (void)
 }
 
 static gboolean
-contents_valid_accumulator (GSignalInvocationHint *ihint,
+contents_valid_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
 			    GValue *return_accu,
 			    const GValue *handler_return,
-			    gpointer data)
+			    G_GNUC_UNUSED gpointer data)
 {
         gboolean thisvalue;
 
@@ -72,7 +74,7 @@ contents_valid_accumulator (GSignalInvocationHint *ihint,
 }
 
 static gboolean
-m_class_contents_valid (GdauiDataEntry *de, GError **error)
+m_class_contents_valid (G_GNUC_UNUSED GdauiDataEntry *de, G_GNUC_UNUSED GError **error)
 {
 	return TRUE;
 }
