@@ -1,31 +1,30 @@
-/* gdaui-rt-editor.c
- *
+/*
  * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
-#include "gdaui-rt-editor.h"
 #include <glib/gi18n-lib.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 #include "bullet.h"
 #include "bulleth.h"
 #define MAX_BULLETS 2
 //gchar * bullet_strings[] = {"•", "◦"};
+#include <libgda-ui.h>
 
 GdkPixbuf *bullet_pix[MAX_BULLETS] = {NULL};
 gchar     *lists_tokens[MAX_BULLETS] = {"- ", " - "};
@@ -675,6 +674,8 @@ add_image_cb (G_GNUC_UNUSED GtkAction *action, GdauiRtEditor *rte)
                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                            NULL);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg),
+					     gdaui_get_default_path ());
         filter = gtk_file_filter_new ();
         gtk_file_filter_add_pixbuf_formats (filter);
         gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dlg), filter);
@@ -684,6 +685,7 @@ add_image_cb (G_GNUC_UNUSED GtkAction *action, GdauiRtEditor *rte)
                 GError *error = NULL;
 
                 filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
+		gdaui_set_default_path (gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg)));
 
 		GdkPixbuf *pixbuf;
 		pixbuf = gdk_pixbuf_new_from_file (filename, &error);
@@ -1049,7 +1051,14 @@ apply_markup (GdauiRtEditor *rte, GtkTextBuffer *textbuffer, TextTag *current, G
 		pe = start;
 		data = remove_newlines_from_base64 (gtk_text_buffer_get_text (textbuffer, &ps, &pe, FALSE));
 		/*g_print ("{{{%s}}}\n", data);*/
+#if GLIB_CHECK_VERSION(2,20,0)
 		g_base64_decode_inplace (data, &length);
+#else
+		gchar *tmp;
+		tmp = g_base64_decode (data, &length);
+		g_free (data);
+		data = tmp;
+#endif
 		if (gdk_pixdata_deserialize (&pixdata, length, (guint8*) data, NULL)) {
 			GdkPixbuf *pixbuf;
 			pixbuf = gdk_pixbuf_from_pixdata (&pixdata, TRUE, NULL);
@@ -1241,7 +1250,7 @@ spaces_since_start_of_line (GtkTextIter *iter)
  * get_markup_token
  * @iter: starting position
  * @out_nb_spaces_before: a place to set the value returned by spaces_since_start_of_line() if called
- * @out_end: place to put the last used position, or %NULL
+ * @out_end: (allow-none): place to put the last used position, or %NULL
  * @rte: the #GdauiRtEditor
  *
  * Parses marking tokens, nothing else
@@ -1898,6 +1907,7 @@ gdaui_rt_editor_set_editable (GdauiRtEditor *editor, gboolean editable)
 {
 	g_return_if_fail (GDAUI_IS_RT_EDITOR (editor));
 	gtk_text_view_set_editable (editor->priv->textview, editable);
+	gtk_text_view_set_cursor_visible (editor->priv->textview, editable);
 	show_hide_toolbar (editor);
 }
 
