@@ -1,21 +1,21 @@
-/* gdaui-server-operation.c
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2006 - 2009 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
@@ -170,7 +170,8 @@ gdaui_server_operation_get_type (void)
 			NULL,
 			sizeof (GdauiServerOperation),
 			0,
-			(GInstanceInitFunc) gdaui_server_operation_init
+			(GInstanceInitFunc) gdaui_server_operation_init,
+			0
 		};
 
 		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiServerOperation", &info, 0);
@@ -643,14 +644,14 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 					if (label_entry) {
 						parent = gtk_widget_get_parent (label_entry);
 						if (parent) {
-							gtk_widget_ref (label_entry);
+							g_object_ref (label_entry);
 							gtk_container_remove (GTK_CONTAINER (parent), label_entry);
 						}
 						gtk_table_attach (GTK_TABLE (table), label_entry,
 								  0, 1, tab_index, tab_index+1,
 								  GTK_FILL | GTK_SHRINK, GTK_SHRINK, 0, 0);
 						if (parent)
-							gtk_widget_unref (label_entry);
+							g_object_unref (label_entry);
 					}
 					nb_labels++;
 					tab_index++;
@@ -939,7 +940,7 @@ sequence_item_added_cb (GdaServerOperation *op, const gchar *seq_path, gint item
 			}
 
 			/* move children DOWN if necessary and change the "_index" property */
-			if (top_attach >= item_index) {
+			if (top_attach >= (guint)item_index) {
 				struct MoveChild *mc;
 				gint index;
 
@@ -1015,7 +1016,7 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 			guint top_attach;
 			gtk_container_child_get (GTK_CONTAINER (table), child,
 						 "top-attach", &top_attach, NULL);
-			if (top_attach == item_index) {
+			if (top_attach == (guint)item_index) {
 				gtk_widget_destroy (child);
 				g_list_free (children);
 				children = gtk_container_get_children (GTK_CONTAINER (table));
@@ -1045,7 +1046,7 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 			}
 
 			/* move widgets UP if necessary and change the "_index" property */
-			if (top_attach > item_index) {
+			if (top_attach > (guint)item_index) {
 				struct MoveChild *mc;
 				gint index;
 
@@ -1082,9 +1083,9 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 /**
  * gdaui_server_operation_new_in_dialog
  * @op: a #GdaServerOperation object
- * @parent: the parent window for the new dialog, or %NULL
- * @title: the title of the dialog window, or %NULL
- * @header: a helper text displayed at the top of the dialog, or %NULL
+ * @parent: (allow-none): the parent window for the new dialog, or %NULL
+ * @title: (allow-none): the title of the dialog window, or %NULL
+ * @header: (allow-none): a helper text displayed at the top of the dialog, or %NULL
  *
  * Creates a new #GdauiServerOperation widget in the same way as gdaui_server_operation_new()
  * and puts it into a #GtkDialog widget. The returned dialog has the "Ok" and "Cancel" buttons
@@ -1152,7 +1153,8 @@ static void create_table_grid_fields_iter_row_changed_cb (GdaDataModelIter *grid
 static void create_table_proxy_row_inserted_cb (GdaDataProxy *proxy, gint row, GdauiServerOperation *form);
 static GtkWidget *
 create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar *path,
-					 gchar **section_str, GSList **label_widgets)
+					 G_GNUC_UNUSED gchar **section_str,
+					 G_GNUC_UNUSED GSList **label_widgets)
 {
 	GdaServerOperationNode *info_node;
 	GtkWidget *hlayout, *sw, *box, *label;
