@@ -1,23 +1,25 @@
-/* GDA library
- * Copyright (C) 2006 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2006 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 Brecht Sanders <brecht@edustria.be>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gda-data-model-import.h"
@@ -786,6 +788,8 @@ gda_data_model_import_get_property (GObject *object,
  *   <listitem><para>Other formats: no option</para></listitem>
  * </itemizedlist>
  *
+ * Note: after the creation, please use gda_data_model_import_get_errors() to check any error.
+ *
  * Returns: (transfer full): a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
@@ -1154,8 +1158,7 @@ clean_field_specs (GSList *fields)
 	GSList *list;
 	XmlColumnSpec *spec;
 
-	list = fields;
-	while (list) {
+	for (list = fields; list; list = list->next) {
 		spec = (XmlColumnSpec*)(list->data);
 		xmlFree (spec->id);
 		xmlFree (spec->name);
@@ -1165,8 +1168,6 @@ clean_field_specs (GSList *fields)
 		xmlFree (spec->table);
 		xmlFree (spec->ref);
 		xmlFree (spec);
-
-		list = g_slist_next (list);
 	}
 	g_slist_free (fields);
 }
@@ -1316,10 +1317,10 @@ init_xml_import (GdaDataModelImport *model)
 			model->priv->extract.xml.reader = NULL;
 			return;
 		}
-		
-		list = fields;
-		pos = 0;
-		while (list) {
+
+		for (list = fields, pos = 0;
+		     list;
+		     list = list->next, pos++) {
 			GdaColumn *column;
 			XmlColumnSpec *spec;
 			
@@ -1332,9 +1333,6 @@ init_xml_import (GdaDataModelImport *model)
 			gda_column_set_dbms_type (column, (gchar*)spec->dbms_type);
 			gda_column_set_g_type (column, spec->gdatype);
 			gda_column_set_allow_null (column, spec->nullok);
-			
-			list = g_slist_next (list);
-			pos++;
 		}
 		clean_field_specs (fields);
 
@@ -1653,9 +1651,9 @@ init_node_import (GdaDataModelImport *model)
 	if (str) 
 		g_object_set_data_full (G_OBJECT (model), "descr", str, xmlFree);
 
-	list = fields;
-	pos = 0;
-	while (list) {
+	for (list = fields, pos = 0;
+	     list;
+	     list = list->next, pos++) {
 		GdaColumn *column;
 		XmlColumnSpec *spec;
 
@@ -1669,9 +1667,6 @@ init_node_import (GdaDataModelImport *model)
 		gda_column_set_allow_null (column, spec->nullok);
 
 		model->priv->columns = g_slist_prepend (model->priv->columns, gda_column_copy (column));
-
-		list = g_slist_next (list);
-		pos++;
 	}
 	clean_field_specs (fields);
 	model->priv->columns = g_slist_reverse (model->priv->columns);
