@@ -1,21 +1,21 @@
-/* gda-meta-struct-io.c
+/*
+ * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2008 - 2010 Vivien Malerba <malerba@gnome-db.org>
  *
- * Copyright (C) 2008 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -31,16 +31,22 @@
 
 static GdaMetaDbObject *create_table_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar *quoted_catalog,
 					     const GValue *schema, const gchar *quoted_schema, xmlNodePtr node, GError **error);
+/*
 static GdaMetaDbObject *create_view_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar *quoted_catalog,
 					    const GValue *schema, const gchar *quoted_schema, xmlNodePtr node, GError **error);
+*/
 
 
 
 /**
- * gda_meta_struct_load_from_xml_file
- * @mstruct:
- * @xml_spec_file:
- * @error:
+ * gda_meta_struct_load_from_xml_file:
+ * @mstruct: a #GdaMetaStruct object
+ * @catalog: (allow-none): the catalog name, or %NULL
+ * @schema: (allow-none): the schema name, or %NULL
+ * @xml_spec_file: the specifications as the name of an XML file
+ * @error: (allow-none): a place to store errors, or %NULL
+ *
+ * Loads an XML description into @mstruct
  *
  * returns: TRUE if no error has occurred
  */
@@ -62,17 +68,11 @@ gda_meta_struct_load_from_xml_file (GdaMetaStruct *mstruct, const gchar *catalog
 
 	if (catalog) {
 		g_value_set_string ((catalog_value = gda_value_new (G_TYPE_STRING)), catalog);
-		if (gda_sql_identifier_needs_quotes (catalog)) 
-			quoted_catalog = gda_sql_identifier_add_quotes (catalog);
-		else
-			quoted_catalog = g_strdup (catalog);
+		quoted_catalog = gda_sql_identifier_quote (catalog, NULL, NULL, FALSE, FALSE);
 	}
 	if (schema) {
 		g_value_set_string ((schema_value = gda_value_new (G_TYPE_STRING)), schema);
-		if (gda_sql_identifier_needs_quotes (schema)) 
-			quoted_schema = gda_sql_identifier_add_quotes (schema);
-		else
-			quoted_schema = g_strdup (schema);
+		quoted_schema = gda_sql_identifier_quote (schema, NULL, NULL, FALSE, FALSE);
 	}
 
 	/* load information schema's structure XML file */
@@ -168,11 +168,12 @@ create_table_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar
 	xmlChar *table_name, *table_schema;
 	GString *full_table_name;
 	GValue *v1 = NULL, *v2 = NULL, *v3 = NULL;
+	gchar *tmp;
 
 	table_name = xmlGetProp (node, BAD_CAST "name");
 	if (!table_name) {
 		g_set_error (error, GDA_META_STRUCT_ERROR, 0, /* FIXME */
-			     _("Missing table name from <table> node"));
+			     "%s", _("Missing table name from <table> node"));
 		return NULL;
 	}
 	table_schema = xmlGetProp (node, BAD_CAST "schema");
@@ -186,13 +187,9 @@ create_table_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar
 	}
 	if (table_schema) {
 		g_value_set_string ((v2 = gda_value_new (G_TYPE_STRING)), (gchar *) table_schema);
-		if (gda_sql_identifier_needs_quotes ((gchar *) table_schema)) {
-			gchar *tmp = gda_sql_identifier_add_quotes ((gchar *) table_schema);
-			g_string_append (full_table_name, tmp);
-			g_free (tmp);
-		}
-		else
-			g_string_append (full_table_name, (gchar*) table_schema);
+		tmp = gda_sql_identifier_quote ((gchar *) table_schema, NULL, NULL, FALSE, FALSE);
+		g_string_append (full_table_name, tmp);
+		g_free (tmp);
 		g_string_append_c (full_table_name, '.');
 	}
 	else if (quoted_schema) {
@@ -201,13 +198,10 @@ create_table_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar
 		g_string_append_c (full_table_name, '.');
 	}
 	g_value_set_string ((v3 = gda_value_new (G_TYPE_STRING)), (gchar *) table_name);
-	if (gda_sql_identifier_needs_quotes ((gchar *) table_name)) {
-		gchar *tmp = gda_sql_identifier_add_quotes ((gchar *) table_name);
-		g_string_append (full_table_name, tmp);
-		g_free (tmp);
-	}
-	else
-		g_string_append (full_table_name, (gchar *) table_name);
+	
+	tmp = gda_sql_identifier_quote ((gchar *) table_name, NULL, NULL, FALSE, FALSE);
+	g_string_append (full_table_name, tmp);
+	g_free (tmp);
 
 	/* a new GdaMetaDbObject structure */
 	dbobj = g_new0 (GdaMetaDbObject, 1);
@@ -276,7 +270,7 @@ create_table_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar
 			if (extra) {
 				GValue *true_value;
 				g_value_set_boolean ((true_value = gda_value_new (G_TYPE_BOOLEAN)), TRUE);
-				gda_meta_table_column_set_attribute (tcol, GDA_ATTRIBUTE_AUTO_INCREMENT, true_value);
+				gda_meta_table_column_set_attribute_static (tcol, GDA_ATTRIBUTE_AUTO_INCREMENT, true_value);
 				gda_value_free (true_value);
 				xmlFree (extra);
 			}
@@ -406,6 +400,7 @@ create_table_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar
 	return NULL;
 }
 
+/*
 static GdaMetaDbObject *
 create_view_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar *quoted_catalog,
 		    const GValue *schema, const gchar *quoted_schema, xmlNodePtr node, GError **error)
@@ -413,4 +408,5 @@ create_view_object (GdaMetaStruct *mstruct, const GValue *catalog, const gchar *
 	TO_IMPLEMENT;
 	return NULL;
 }
+*/
 
