@@ -1,25 +1,31 @@
-/* GDA library
- * Copyright (C) 1998 - 2008 The GNOME Foundation.
+/*
+ * Copyright (C) 2000 - 2003 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2001 Cleber Rodrigues <cleberrrjr@bol.com.br>
+ * Copyright (C) 2002 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * Copyright (C) 2003 Laurent Sansonetti <laurent@datarescue.be>
+ * Copyright (C) 2003 Paisa Seeluangsawat <paisa@users.sf.net>
+ * Copyright (C) 2003 Xabier Rodrguez Calvar <xrcalvar@igalia.com>
+ * Copyright (C) 2004 Paisa  Seeluangsawat <paisa@users.sf.net>
+ * Copyright (C) 2004 Szalai Ferenc <szferi@einstein.ki.iif.hu>
+ * Copyright (C) 2005 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *	Rodrigo Moya <rodrigo@gnome-db.org>
- *	lvaro Pea <alvaropg@telefonica.net>
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gda-row.h"
@@ -79,7 +85,7 @@ gda_row_class_init (GdaRowClass *klass)
 }
 
 static void
-gda_row_init (GdaRow *row, GdaRowClass *klass)
+gda_row_init (GdaRow *row, G_GNUC_UNUSED GdaRowClass *klass)
 {
 	g_return_if_fail (GDA_IS_ROW (row));
 	
@@ -138,7 +144,7 @@ gda_row_set_property (GObject *object,
 			row->priv->fields = g_new0 (GValue, row->priv->nfields);			
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
                 }
         }
@@ -159,7 +165,7 @@ gda_row_get_property (GObject *object,
 			g_value_set_int (value, row->priv->nfields);
 			break;
 		default:
-			g_assert_not_reached ();
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
                 }
         }
@@ -181,7 +187,8 @@ gda_row_get_type (void)
 			NULL,
 			sizeof (GdaRow),
 			0,
-			(GInstanceInitFunc) gda_row_init
+			(GInstanceInitFunc) gda_row_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -193,7 +200,7 @@ gda_row_get_type (void)
 }
 
 /**
- * gda_row_new
+ * gda_row_new:
  * @count: number of #GValue in the new #GdaRow.
  *
  * Creates a #GdaRow which can hold @count #GValue values.
@@ -209,7 +216,7 @@ gda_row_new (gint count)
 }
 
 /**
- * gda_row_get_value
+ * gda_row_get_value:
  * @row: a #GdaRow
  * @num: field index.
  *
@@ -218,7 +225,7 @@ gda_row_new (gint count)
  * This is a pointer to the internal array of values. Don't try to free
  * or modify it (modifying is reserved to database provider's implementations).
  *
- * Returns: a pointer to the #GValue in the position @num of @row.
+ * Returns: (transfer none): a pointer to the #GValue in the position @num of @row.
  */
 GValue *
 gda_row_get_value (GdaRow *row, gint num)
@@ -230,7 +237,7 @@ gda_row_get_value (GdaRow *row, gint num)
 }
 
 /**
- * gda_row_invalidate_value
+ * gda_row_invalidate_value:
  * @row: a #GdaRow
  * @value: a #GValue belonging to @row (obtained with gda_row_get_value()).
  * 
@@ -238,14 +245,14 @@ gda_row_get_value (GdaRow *row, gint num)
  * providers' implementations to report any error while reading a value from the database.
  */
 void
-gda_row_invalidate_value (GdaRow *row, GValue *value)
+gda_row_invalidate_value (G_GNUC_UNUSED GdaRow *row, GValue *value)
 {
 	gda_value_set_null (value);
 	G_VALUE_TYPE (value) = G_TYPE_NONE;
 }
 
 /**
- * gda_row_value_is_valid
+ * gda_row_value_is_valid:
  * @row: a #GdaRow.
  * @value: a #GValue belonging to @row (obtained with gda_row_get_value()).
  *
@@ -253,16 +260,16 @@ gda_row_invalidate_value (GdaRow *row, GValue *value)
  * This method is mainly used by database
  * providers' implementations to report any error while reading a value from the database.
  *
- * Returns: TRUE if @value is valid
+ * Returns: %TRUE if @value is valid
  */
 gboolean
-gda_row_value_is_valid (GdaRow *row, GValue *value)
+gda_row_value_is_valid (G_GNUC_UNUSED GdaRow *row, GValue *value)
 {
 	return (G_VALUE_TYPE (value) == G_TYPE_NONE) ? FALSE : TRUE;
 }
 
 /**
- * gda_row_get_length
+ * gda_row_get_length:
  * @row: a #GdaRow.
  *
  * Returns: the number of columns that the @row has.
