@@ -1,23 +1,23 @@
-/* GDA library
- * Copyright (C) 2006 - 2009 The GNOME Foundation.
+/*
+ * Copyright (C) 2006 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2006 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #ifndef __GDA_SERVER_OPERATION_H__
@@ -66,6 +66,27 @@ typedef enum {
 	GDA_SERVER_OPERATION_LAST
 } GdaServerOperationType;
 
+/* error reporting */
+extern GQuark gda_server_operation_error_quark (void);
+#define GDA_SERVER_OPERATION_ERROR gda_server_operation_error_quark ()
+
+typedef enum {
+	GDA_SERVER_OPERATION_OBJECT_NAME_ERROR,
+	GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR
+} GdaServerOperationError;
+
+typedef enum
+{
+	GDA_SERVER_OPERATION_CREATE_TABLE_NOTHING_FLAG   = 1 << 0,
+	GDA_SERVER_OPERATION_CREATE_TABLE_PKEY_FLAG      = 1 << 1,
+	GDA_SERVER_OPERATION_CREATE_TABLE_NOT_NULL_FLAG  = 1 << 2,
+	GDA_SERVER_OPERATION_CREATE_TABLE_UNIQUE_FLAG    = 1 << 3,
+	GDA_SERVER_OPERATION_CREATE_TABLE_AUTOINC_FLAG   = 1 << 4,
+	GDA_SERVER_OPERATION_CREATE_TABLE_FKEY_FLAG      = 1 << 5,
+	/* Flags combinations */
+	GDA_SERVER_OPERATION_CREATE_TABLE_PKEY_AUTOINC_FLAG = GDA_SERVER_OPERATION_CREATE_TABLE_PKEY_FLAG | GDA_SERVER_OPERATION_CREATE_TABLE_AUTOINC_FLAG
+} GdaServerOperationCreateTableFlag;
+
 typedef enum {
 	GDA_SERVER_OPERATION_NODE_PARAMLIST,
 	GDA_SERVER_OPERATION_NODE_DATA_MODEL,
@@ -121,11 +142,17 @@ GdaServerOperationType     gda_server_operation_string_to_op_type       (const g
 GdaServerOperationNode    *gda_server_operation_get_node_info           (GdaServerOperation *op, const gchar *path_format, ...);
 
 const GValue              *gda_server_operation_get_value_at            (GdaServerOperation *op, const gchar *path_format, ...);
+const GValue              *gda_server_operation_get_value_at_path       (GdaServerOperation *op, const gchar *path);
 gchar                     *gda_server_operation_get_sql_identifier_at   (GdaServerOperation *op,
 									 GdaConnection *cnc, GdaServerProvider *prov,
-									 const gchar *path_format, ...);
+									 const gchar *path_format, ...);    
+gchar                     *gda_server_operation_get_sql_identifier_at_path (GdaServerOperation *op, 
+									    GdaConnection *cnc, GdaServerProvider *prov,
+									    const gchar *path);
 gboolean                   gda_server_operation_set_value_at            (GdaServerOperation *op, const gchar *value, 
 									 GError **error, const gchar *path_format, ...);
+gboolean                   gda_server_operation_set_value_at_path       (GdaServerOperation *op, const gchar *value, 
+									 const gchar *path, GError **error);
 
 xmlNodePtr                 gda_server_operation_save_data_to_xml        (GdaServerOperation *op, GError **error);
 gboolean                   gda_server_operation_load_data_from_xml      (GdaServerOperation *op, 
@@ -148,6 +175,22 @@ gboolean                   gda_server_operation_del_item_from_sequence  (GdaServ
 
 gboolean                   gda_server_operation_is_valid                (GdaServerOperation *op, const gchar *xml_file, GError **error);
 
+/*
+ * Database creation and destruction
+ */
+GdaServerOperation *gda_server_operation_prepare_create_database       (const gchar *provider, const gchar *db_name, GError **error);
+gboolean            gda_server_operation_perform_create_database       (GdaServerOperation *op, const gchar *provider, GError **error);
+GdaServerOperation *gda_server_operation_prepare_drop_database         (const gchar *provider, const gchar *db_name, GError **error);
+gboolean            gda_server_operation_perform_drop_database         (GdaServerOperation *op, const gchar *provider, GError **error);
+
+/*
+ * Tables creation and destruction
+ */
+GdaServerOperation *gda_server_operation_prepare_create_table	      (GdaConnection *cnc, const gchar *table_name, GError **error, ...);
+gboolean            gda_server_operation_perform_create_table          (GdaServerOperation *op, GError **error);
+GdaServerOperation *gda_server_operation_prepare_drop_table            (GdaConnection *cnc, const gchar *table_name, GError **error);
+gboolean            gda_server_operation_perform_drop_table            (GdaServerOperation *op, GError **error);
+
 G_END_DECLS
 
 #endif
