@@ -1,21 +1,23 @@
-/* gda-set.c
+/*
+ * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2003 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdarg.h>
@@ -193,7 +195,8 @@ gda_set_get_type (void)
 			NULL,
 			sizeof (GdaSet),
 			0,
-			(GInstanceInitFunc) gda_set_init
+			(GInstanceInitFunc) gda_set_init,
+			0
 		};
 		
 		g_static_mutex_lock (&registering);
@@ -206,10 +209,10 @@ gda_set_get_type (void)
 }
 
 static gboolean
-validate_accumulator (GSignalInvocationHint *ihint,
+validate_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
 		      GValue *return_accu,
 		      const GValue *handler_return,
-		      gpointer data)
+		      G_GNUC_UNUSED gpointer data)
 {
 	GError *error;
 
@@ -220,13 +223,14 @@ validate_accumulator (GSignalInvocationHint *ihint,
 }
 
 static GError *
-m_validate_holder_change (GdaSet *set, GdaHolder *holder, const GValue *new_value)
+m_validate_holder_change (G_GNUC_UNUSED GdaSet *set, G_GNUC_UNUSED GdaHolder *holder,
+			  G_GNUC_UNUSED const GValue *new_value)
 {
 	return NULL;
 }
 
 static GError *
-m_validate_set (GdaSet *set)
+m_validate_set (G_GNUC_UNUSED GdaSet *set)
 {
 	return NULL;
 }
@@ -256,7 +260,7 @@ gda_set_class_init (GdaSetClass *class)
 	 * Gets emitted when a #GdaHolder's in @set is going to change its value. One can connect to
 	 * this signal to control which values @holder can have (for example to implement some business rules)
 	 *
-	 * Return value: NULL if @holder is allowed to change its value to @new_value, or a #GError
+	 * Returns: NULL if @holder is allowed to change its value to @new_value, or a #GError
 	 * otherwise.
 	 */
 	gda_set_signals[VALIDATE_HOLDER_CHANGE] =
@@ -274,7 +278,7 @@ gda_set_class_init (GdaSetClass *class)
 	 * Gets emitted when gda_set_is_valid() is called, use
 	 * this signal to control which combination of values @set's holder can have (for example to implement some business rules)
 	 *
-	 * Return value: NULL if @set's contents has been validated, or a #GError
+	 * Returns: NULL if @set's contents has been validated, or a #GError
 	 * otherwise.
 	 */
 	gda_set_signals[VALIDATE_SET] =
@@ -450,7 +454,7 @@ gda_set_new_read_only (GSList *holders)
  *
  * Creates a new #GdaSet object, copy of @set
  *
- * Returns: a new #GdaSet object
+ * Returns: (transfer full): a new #GdaSet object
  */
 GdaSet *
 gda_set_copy (GdaSet *set)
@@ -471,7 +475,7 @@ gda_set_copy (GdaSet *set)
 }
 
 /**
- * gda_set_new_inline:
+ * gda_set_new_inline: (skip)
  * @nb: the number of value holders which will be contained in the new #GdaSet
  * @...: a serie of a (const gchar*) id, (GType) type, and value
  *
@@ -483,7 +487,7 @@ gda_set_copy (GdaSet *set)
  * to use an unsupported type will result in a warning, and the returned value holder holding a safe default
  * value.
  *
- * Returns: a new #GdaSet object
+ * Returns: (transfer full): a new #GdaSet object
  */ 
 GdaSet *
 gda_set_new_inline (gint nb, ...)
@@ -546,6 +550,10 @@ gda_set_new_inline (gint nb, ...)
 			g_value_set_ulong (value, va_arg (ap, gulong));
 		else if (type == G_TYPE_GTYPE)
 			g_value_set_gtype (value, va_arg(ap, GType));
+		else if (type == GDA_TYPE_TIMESTAMP)
+			gda_value_set_timestamp (value, va_arg(ap, GdaTimestamp *));
+		else if (type == GDA_TYPE_TIME)
+			gda_value_set_time (value, va_arg(ap, GdaTime *));
 		else {
 			g_warning (_("%s() does not handle values of type '%s'."),
 				   __FUNCTION__, g_type_name (type));
@@ -578,15 +586,15 @@ gda_set_new_inline (gint nb, ...)
 }
 
 /**
- * gda_set_set_holder_value:
+ * gda_set_set_holder_value: (skip)
  * @set: a #GdaSet object
- * @error: a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
  * @...: value, of the correct type, depending on the requested holder's type (not NULL)
  *
  * Set the value of the #GdaHolder which ID is @holder_id to a specified value
  *
- * Returns: TRUE if no error occurred and the value was set correctly
+ * Returns: %TRUE if no error occurred and the value was set correctly
  */
 gboolean
 gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, ...)
@@ -665,7 +673,7 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
  *
  * Get the value of the #GdaHolder which ID is @holder_id
  *
- * Returns: the requested GValue, or %NULL (see gda_holder_get_value())
+ * Returns: (transfer none): the requested GValue, or %NULL (see gda_holder_get_value())
  */
 const GValue *
 gda_set_get_holder_value (GdaSet *set, const gchar *holder_id)
@@ -708,12 +716,12 @@ xml_validity_error_func (void *ctx, const char *msg, ...)
 /**
  * gda_set_new_from_spec_string:
  * @xml_spec: a string
- * @error: a place to store the error, or %NULL
+ * @error: (allow-none): a place to store the error, or %NULL
  *
  * Creates a new #GdaSet object from the @xml_spec
  * specifications
  *
- * Returns: a new object, or %NULL if an error occurred
+ * Returns: (transfer full): a new object, or %NULL if an error occurred
  */
 GdaSet *
 gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
@@ -800,12 +808,12 @@ gda_set_new_from_spec_string (const gchar *xml_spec, GError **error)
 /**
  * gda_set_new_from_spec_node:
  * @xml_spec: a #xmlNodePtr for a &lt;holders&gt; tag
- * @error: a place to store the error, or %NULL
+ * @error: (allow-none): a place to store the error, or %NULL
  *
  * Creates a new #GdaSet object from the @xml_spec
  * specifications
  *
- * Returns: a new object, or %NULL if an error occurred
+ * Returns: (transfer full): a new object, or %NULL if an error occurred
  */
 GdaSet *
 gda_set_new_from_spec_node (xmlNodePtr xml_spec, GError **error)
@@ -1010,7 +1018,7 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 }
 
 static void
-source_changed_holder_cb (GdaHolder *holder, GdaSet *set)
+source_changed_holder_cb (G_GNUC_UNUSED GdaHolder *holder, GdaSet *set)
 {
 	compute_public_data (set);
 }
@@ -1060,7 +1068,7 @@ changed_holder_cb (GdaHolder *holder, GdaSet *set)
 }
 
 static void
-group_free (GdaSetGroup *group, gpointer data)
+group_free (GdaSetGroup *group, G_GNUC_UNUSED gpointer data)
 {
 	g_slist_free (group->nodes);
 	g_free (group);
@@ -1390,7 +1398,7 @@ set_remove_source (GdaSet *set, GdaSetSource *source)
 /**
  * gda_set_is_valid:
  * @set: a #GdaSet object
- * @error: a place to store validation errors, or %NULL
+ * @error: (allow-none): a place to store validation errors, or %NULL
  *
  * This method tells if all @set's #GdaHolder objects are valid, and if
  * they represent a valid combination of values, as defined by rules
@@ -1472,6 +1480,8 @@ GdaHolder *
 gda_set_get_nth_holder (GdaSet *set, gint pos)
 {
 	g_return_val_if_fail (GDA_IS_SET (set), NULL);
+	g_return_val_if_fail (pos >= 0, NULL);
+
 	if (! set->priv->holders_array) {
 		GSList *list;
 		set->priv->holders_array = g_array_sized_new (FALSE, FALSE, sizeof (GdaHolder*),
@@ -1479,7 +1489,7 @@ gda_set_get_nth_holder (GdaSet *set, gint pos)
 		for (list = set->holders; list; list = list->next)
 			g_array_append_val (set->priv->holders_array, list->data);
 	}
-	if ((pos < 0) || (pos > set->priv->holders_array->len))
+	if ((guint)pos >= set->priv->holders_array->len)
 		return NULL;
 	else
 		return g_array_index (set->priv->holders_array, GdaHolder*, pos);
