@@ -1,21 +1,26 @@
-/* gda-sql-builder.c
+/*
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2009 Johannes Schmid <jhs@gnome.org>
+ * Copyright (C) 2009 - 2010 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2011 Daniel Espinosa <esodan@gmail.com>
  *
- * Copyright (C) 2008 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -478,8 +483,8 @@ gda_sql_builder_set_where (GdaSqlBuilder *builder, GdaSqlBuilderId cond_id)
  * gda_sql_builder_select_add_field:
  * @builder: a #GdaSqlBuilder object
  * @field_name: a field name
- * @table_name: a table name, or %NULL
- * @alias: an alias (eg. for the "AS" clause), or %NULL
+ * @table_name: (allow-none): a table name, or %NULL
+ * @alias: (allow-none): an alias (eg. for the "AS" clause), or %NULL
  *
  * Valid only for: SELECT statements.
  *
@@ -503,8 +508,11 @@ gda_sql_builder_select_add_field (GdaSqlBuilder *builder, const gchar *field_nam
 	}
 	g_return_val_if_fail (field_name && *field_name, 0);
 
-	if (table_name && *table_name)
+	gboolean tmp_is_allocated = FALSE;
+	if (table_name && *table_name) {
 		tmp = g_strdup_printf ("%s.%s", table_name, field_name);
+		tmp_is_allocated = TRUE;
+	}
 	else
 		tmp = (gchar*) field_name;
 
@@ -517,7 +525,7 @@ gda_sql_builder_select_add_field (GdaSqlBuilder *builder, const gchar *field_nam
 		gda_sql_builder_add_field_value_id (builder,
 					      field_id,
 					      0);
-	if (table_name)
+	if (tmp_is_allocated)
 		g_free (tmp);
 
     return field_id;
@@ -643,7 +651,7 @@ gda_sql_builder_add_field_value (GdaSqlBuilder *builder, const gchar *field_name
  * gda_sql_builder_add_field_value_as_gvalue:
  * @builder: a #GdaSqlBuilder object
  * @field_name: a field name
- * @value: value to set the field to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
+ * @value: (allow-none): value to set the field to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Valid only for: INSERT, UPDATE statements.
  *
@@ -791,8 +799,8 @@ gda_sql_builder_add_field_value_id (GdaSqlBuilder *builder, GdaSqlBuilderId fiel
 /**
  * gda_sql_builder_add_expr_value:
  * @builder: a #GdaSqlBuilder object
- * @dh: a #GdaDataHandler to use, or %NULL
- * @value: value to set the expression to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
+ * @dh: (allow-none): a #GdaDataHandler to use, or %NULL
+ * @value: (allow-none): value to set the expression to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Defines an expression in @builder which may be reused to build other parts of a statement.
  *
@@ -847,7 +855,7 @@ gda_sql_builder_add_expr_value (GdaSqlBuilder *builder, GdaDataHandler *dh, cons
 /**
  * gda_sql_builder_add_expr: (skip)
  * @builder: a #GdaSqlBuilder object
- * @dh: a #GdaDataHandler to use, or %NULL
+ * @dh: (allow-none): a #GdaDataHandler to use, or %NULL
  * @type: the GType of the following argument
  * @...: value to set the expression to, of the type specified by @type
  *
@@ -971,14 +979,17 @@ GdaSqlBuilderId
 gda_sql_builder_add_field_id (GdaSqlBuilder *builder, const gchar *field_name, const gchar *table_name)
 {
 	gchar* tmp = 0;
-	if (table_name && *table_name)
+	gboolean tmp_is_allocated = FALSE;
+	if (table_name && *table_name) {
 		tmp = g_strdup_printf ("%s.%s", table_name, field_name);
+		tmp_is_allocated = TRUE;
+	}
 	else
 		tmp = (gchar*) field_name;
 
 	guint field_id = gda_sql_builder_add_id (builder, tmp);
 
-	if (table_name)
+	if (tmp_is_allocated)
 		g_free (tmp);
 
 	return field_id;
@@ -1680,6 +1691,8 @@ gda_sql_builder_add_function (GdaSqlBuilder *builder, const gchar *func_name, ..
  * Returns: the ID of the new expression, or %0 if there was an error
  *
  * Since: 4.2
+ *
+ * Rename to: gda_sql_builder_add_function
  */
 GdaSqlBuilderId
 gda_sql_builder_add_function_v (GdaSqlBuilder *builder, const gchar *func_name,
@@ -1885,6 +1898,8 @@ gda_sql_builder_add_case (GdaSqlBuilder *builder,
  * Returns: the ID of the new expression, or %0 if there was an error
  *
  * Since: 4.2
+ *
+ * Rename to: gda_sql_builder_add_case
  */
 GdaSqlBuilderId
 gda_sql_builder_add_case_v (GdaSqlBuilder *builder,
