@@ -1,21 +1,24 @@
-/* gda-statement.c
+/*
+ * Copyright (C) 2008 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2008 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * Copyright (C) 2007 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #undef GDA_DISABLE_DEPRECATED
@@ -492,7 +495,30 @@ get_params_foreach_func (GdaSqlAnyPart *node, GdaSet **params, GError **error)
 			      "description", pspec->descr, NULL);
 		gda_holder_set_not_null (h, ! pspec->nullok);
 		if (((GdaSqlExpr*) node)->value) {
-			gda_holder_set_default_value (h, ((GdaSqlExpr*) node)->value);
+			/* Expr's value is "SQL encoded" => we need to convert it to a real value */
+			GValue *evalue;
+			evalue = ((GdaSqlExpr*) node)->value;
+			if (G_VALUE_TYPE (evalue) == G_TYPE_STRING) {
+				GdaDataHandler *dh;
+				GValue *value;
+				dh = gda_data_handler_get_default (pspec->g_type);
+				if (!dh) {
+					g_set_error (error, GDA_STATEMENT_ERROR, GDA_STATEMENT_PARAM_TYPE_ERROR,
+						     _("Can't handle default value of type '%s'"),
+						     gda_g_type_to_string (pspec->g_type));
+					g_object_unref (h);
+					return FALSE;
+				}
+				value = gda_data_handler_get_value_from_sql (dh,
+									     g_value_get_string (evalue),
+									     pspec->g_type);
+				if (!value)
+					value = gda_value_new_default (g_value_get_string (evalue));
+				gda_holder_set_default_value (h, value);
+				gda_value_free (value);
+			}
+			else
+				gda_holder_set_default_value (h, ((GdaSqlExpr*) node)->value);
 			gda_holder_set_value_to_default (h);
 		}
 		gda_set_add_holder (*params, h);
@@ -694,7 +720,7 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 		if (context->provider)
 			dh = gda_server_provider_get_data_handler_g_type (context->provider, context->cnc, G_VALUE_TYPE (value));
 		else  			
-			dh = gda_get_default_handler (G_VALUE_TYPE (value));
+			dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 
 		if (!dh) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
@@ -1301,7 +1327,10 @@ default_render_param_spec (GdaSqlParamSpec *pspec, GdaSqlExpr *expr, GdaSqlRende
 
 		if (flag & GDA_STATEMENT_SQL_PARAMS_LONG) {
 			if (expr->value) {
-				str = context->render_value (expr->value, context, error);
+				if (G_VALUE_TYPE (expr->value) == G_TYPE_STRING)
+					str = g_value_dup_string (expr->value);
+				else
+					str = context->render_value (expr->value, context, error);
 				if (!str) {
 					g_free (quoted_pname);
 					goto err;
@@ -1424,8 +1453,11 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 		if (!str) goto err;
 	}
 	else if (expr->cond) {
-		str = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
-		if (!str) goto err;
+		gchar *tmp;
+		tmp = context->render_operation (GDA_SQL_ANY_PART (expr->cond), context, error);
+		if (!tmp) goto err;
+		str = g_strconcat ("(", tmp, ")", NULL);
+		g_free (tmp);
 	}
 	else if (expr->select) {
 		gchar *str1;
@@ -1439,7 +1471,7 @@ default_render_expr (GdaSqlExpr *expr, GdaSqlRenderingContext *context, gboolean
 
 		if (! GDA_SQL_ANY_PART (expr)->parent ||
 		    (GDA_SQL_ANY_PART (expr)->parent->type != GDA_SQL_ANY_SQL_FUNCTION)) {
-			str = g_strdup_printf ("(%s)", str1);
+			str = g_strconcat ("(", str1, ")", NULL);
 			g_free (str1);
 		}
 		else
@@ -1620,6 +1652,9 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 	case GDA_SQL_OPERATOR_TYPE_LIKE:
 		str = g_strdup_printf ("%s LIKE %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
+	case GDA_SQL_OPERATOR_TYPE_ILIKE:
+		str = g_strdup_printf ("%s ILIKE %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
+		break;
 	case GDA_SQL_OPERATOR_TYPE_GT:
 		str = g_strdup_printf ("%s > %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
 		break;
