@@ -1,25 +1,34 @@
 /*
- * Copyright (C) 1998 - 2011 The GNOME Foundation.
+ * Copyright (C) 2001 - 2003 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * Copyright (C) 2003 - 2004 Laurent Sansonetti <lrz@gnome.org>
+ * Copyright (C) 2003 Paisa Seeluangsawat <paisa@users.sf.net>
+ * Copyright (C) 2004 Caolan McNamara <caolanm@redhat.com>
+ * Copyright (C) 2004 Jrg Billeter <j@bitron.ch>
+ * Copyright (C) 2004 - 2010 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2005 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2009 Armin Burgmeier <armin@openismus.com>
+ * Copyright (C) 2008 - 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2008 Phil Longstaff <plongstaff@rogers.com>
+ * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2011 Daniel Espinosa <esodan@gmail.com>
  *
- * AUTHORS:
- *	Rodrigo Moya <rodrigo@gnome-db.org>
- *	Juan-Mariano de Goyeneche <jmseyas@dit.upm.es> (BLOB issues)
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #undef GDA_DISABLE_DEPRECATED
@@ -55,7 +64,7 @@ extern GdaAttributesManager *gda_holder_attributes_manager;
 #define PROV_CLASS(provider) (GDA_SERVER_PROVIDER_CLASS (G_OBJECT_GET_CLASS (provider)))
 
 /**
- * gda_g_type_to_string
+ * gda_g_type_to_string:
  * @type: Type to convert from.
  *
  * Converts a GType to its string representation (use gda_g_type_from_string() for the
@@ -94,7 +103,7 @@ gda_g_type_to_string (GType type)
 }
 
 /**
- * gda_g_type_from_string
+ * gda_g_type_from_string:
  * @str: the name of a #GType, as returned by gda_g_type_to_string().
  *
  * Converts a named type to ts GType type (also see the gda_g_type_to_string() function).
@@ -150,7 +159,7 @@ gda_g_type_from_string (const gchar *str)
 }
 
 /**
- * gda_default_escape_string
+ * gda_default_escape_string:
  * @string: string to escape
  *
  * Escapes @string to make it understandable by a DBMS. The escape method is very common and replaces any
@@ -204,7 +213,7 @@ gda_default_escape_string (const gchar *string)
 }
 
 /**
- * gda_default_unescape_string
+ * gda_default_unescape_string:
  * @string: string to unescape
  *
  * Do the reverse of gda_default_escape_string(): transforms any "''" into "'", any
@@ -265,14 +274,65 @@ gda_default_unescape_string (const gchar *string)
 }
 
 /**
- * gda_utility_check_data_model
+ * gda_utility_check_data_model_v:
+ * @model: a #GdaDataModel object
+ * @nbcols: the minimum requested number of columns
+ * @types: (array length=nbcols): array with @nbcols length of type GType or null (if any data type is accepted)
+ *
+ * Check the column types of a GdaDataModel.
+ *
+ * Returns: %TRUE if the data model's columns match the provided data types and number
+ *
+ * Since: 4.2.6
+ *
+ * Rename to: gda_utility_check_data_model
+ */
+gboolean
+gda_utility_check_data_model_v (GdaDataModel *model, gint nbcols, GType* types)
+{
+	gboolean retval = TRUE;
+	gint i;
+
+	g_return_val_if_fail (model && GDA_IS_DATA_MODEL (model), FALSE);
+
+	/* number of columns */
+	if (gda_data_model_get_n_columns (model) < nbcols)
+		return FALSE;
+
+	/* type of each column */
+	if (nbcols > 0) {
+		GdaColumn *att;
+		GType mtype, rtype;
+		i = 0;
+		while ((i<nbcols) && retval) {
+			att = gda_data_model_describe_column (model, i);
+			mtype = gda_column_get_g_type (att);
+
+			rtype = types [i];
+			if (mtype != rtype) {
+				retval = FALSE;
+#ifdef GDA_DEBUG_NO
+				g_print ("Column %d: Expected %s, got %s\n",
+					 i, gda_g_type_to_string (rtype), gda_g_type_to_string (mtype));
+#endif
+			}
+			i++;
+		}
+	}
+
+	return retval;
+}
+
+
+/**
+ * gda_utility_check_data_model: (skip)
  * @model: a #GdaDataModel object
  * @nbcols: the minimum requested number of columns
  * @Varargs: @nbcols arguments of type GType or -1 (if any data type is accepted)
  *
  * Check the column types of a GdaDataModel.
  *
- * Returns: TRUE if the data model's columns match the provided data types and number
+ * Returns: %TRUE if the data model's columns match the provided data types and number
  */
 gboolean
 gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
@@ -321,12 +381,12 @@ gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 }
 
 /**
- * gda_utility_data_model_dump_data_to_xml
+ * gda_utility_data_model_dump_data_to_xml:
  * @model: a #GdaDataModel
  * @parent: the parent XML node
- * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (allow-none) (array length=nb_cols): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (allow-none) (array length=nb_rows): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
  * @use_col_ids: set to %TRUE to add column ID information
  *
@@ -337,7 +397,7 @@ gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
  * to access data in @model previously to calling this method, and this iterator will be moved (point to
  * another row).
  *
- * Returns: TRUE if no error occurred
+ * Returns: %TRUE if no error occurred
  */
 gboolean
 gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
@@ -477,7 +537,7 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 
 
 /**
- * gda_utility_data_model_find_column_description
+ * gda_utility_data_model_find_column_description:
  * @model: a #GdaDataSelect data model
  * @field_name: field name
  *
@@ -531,7 +591,7 @@ gda_utility_data_model_find_column_description (GdaDataSelect *model, const gcha
 }
 
 /**
- * gda_utility_holder_load_attributes
+ * gda_utility_holder_load_attributes:
  * @holder: a #GdaHolder
  * @node: an xmlNodePtr with a &lt;parameter&gt; tag
  * @sources: a list of #GdaDataModel
@@ -705,7 +765,7 @@ gda_utility_holder_load_attributes (GdaHolder *holder, xmlNodePtr node, GSList *
 
 #define GDA_PARAM_ENCODE_TOKEN "__gda"
 /**
- * gda_text_to_alphanum
+ * gda_text_to_alphanum:
  * @text: the text to convert
  *
  * The "encoding" consists in replacing non
@@ -740,7 +800,7 @@ gda_text_to_alphanum (const gchar *text)
 }
 
 /**
- * gda_alphanum_to_text
+ * gda_alphanum_to_text:
  * @text: a string
  *
  * Does the opposite of gda_text_to_alphanum(), in the same string 
@@ -830,8 +890,8 @@ dml_statements_check_select_structure (GdaConnection *cnc, GdaSqlStatement *sel_
 }
 
 /**
- * gda_compute_unique_table_row_condition_with_cnc
- * @cnc: a #GdaConnection, or %NULL
+ * gda_compute_unique_table_row_condition_with_cnc:
+ * @cnc: (allow-none): a #GdaConnection, or %NULL
  * @stsel: a #GdaSqlSelectStatement
  * @mtable: a #GdaMetaTable
  * @require_pk: set to TRUE if a primary key ir required
@@ -855,15 +915,14 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 	GdaSqlOperation *and_cond = NULL;
 
 	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), NULL);
-
-	if (mtable->pk_cols_nb == 0) {
-		g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-			     "%s", _("Table does not have any primary key"));
-		return NULL;
-	}
 	
 	expr = gda_sql_expr_new (NULL); /* no parent */
 	if (require_pk) {
+		if (mtable->pk_cols_nb == 0) {
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+				     "%s", _("Table does not have any primary key"));
+			goto onerror;
+		}
 		if (mtable->pk_cols_nb == 0) {
 			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
 					     "%s", _("Table does not have any primary key"));
@@ -879,10 +938,10 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 			GdaMetaTableColumn *tcol;
 			GSList *list;
 			gint index;
-			
+
 			tcol = (GdaMetaTableColumn *) g_slist_nth_data (mtable->columns, mtable->pk_cols_array[i]);
-			for (index = 0, list = stsel->expr_list; 
-			     list; 
+			for (index = 0, list = stsel->expr_list;
+			     list;
 			     index++, list = list->next) {
 				sfield = (GdaSqlSelectField *) list->data;
 				if (sfield->validity_meta_table_column == tcol)
@@ -892,7 +951,7 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 			}
 			if (!sfield) {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
-					     "%s", _("Table's primary key is not selected"));
+					     "%s", _("Table's primary key is not part of SELECT"));
 				goto onerror;
 			}
 			else {
@@ -932,9 +991,74 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 		}
 	}
 	else {
-		TO_IMPLEMENT;
-		gda_sql_expr_free (expr);
-		expr = NULL;
+		GSList *columns;
+		if (! mtable->columns) {
+			g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+					     "%s", _("Table does not have any column"));
+			goto onerror;
+		}
+		else if (mtable->columns->next) {
+			and_cond = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
+			and_cond->operator_type = GDA_SQL_OPERATOR_TYPE_AND;
+			expr->cond = and_cond;
+		}
+		for (columns = mtable->columns; columns; columns = columns->next) {
+			GdaSqlSelectField *sfield = NULL;
+			GdaMetaTableColumn *tcol;
+			GSList *list;
+			gint index;
+
+			tcol = (GdaMetaTableColumn *) columns->data;
+			for (index = 0, list = stsel->expr_list;
+			     list;
+			     index++, list = list->next) {
+				sfield = (GdaSqlSelectField *) list->data;
+				if (sfield->validity_meta_table_column == tcol)
+					break;
+				else
+					sfield = NULL;
+			}
+			if (!sfield) {
+				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
+					     _("Table's column '%s' is not part of SELECT"),
+					     tcol->column_name);
+				goto onerror;
+			}
+			else {
+				GdaSqlOperation *op;
+				GdaSqlExpr *opexpr;
+				GdaSqlParamSpec *pspec;
+
+				/* equal condition */
+				if (and_cond) {
+					opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (and_cond));
+					op = gda_sql_operation_new (GDA_SQL_ANY_PART (opexpr));
+					opexpr->cond = op;
+					and_cond->operands = g_slist_append (and_cond->operands, opexpr);
+				}
+				else {
+					op = gda_sql_operation_new (GDA_SQL_ANY_PART (expr));
+					expr->cond = op;
+				}
+				op->operator_type = GDA_SQL_OPERATOR_TYPE_EQ;
+				/* left operand */
+				gchar *str;
+				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+				str = gda_sql_identifier_quote (tcol->column_name, cnc, NULL, FALSE, FALSE);
+				g_value_take_string (opexpr->value = gda_value_new (G_TYPE_STRING), str);
+
+				op->operands = g_slist_append (op->operands, opexpr);
+
+				/* right operand */
+				opexpr = gda_sql_expr_new (GDA_SQL_ANY_PART (op));
+				pspec = g_new0 (GdaSqlParamSpec, 1);
+				pspec->name = g_strdup_printf ("-%d", index);
+				pspec->g_type = (tcol->gtype != GDA_TYPE_NULL) ? tcol->gtype: G_TYPE_STRING;
+				pspec->nullok = tcol->nullok;
+				opexpr->param_spec = pspec;
+				op->operands = g_slist_append (op->operands, opexpr);
+			}
+		}
 	}
 	return expr;
 	
@@ -944,7 +1068,7 @@ gda_compute_unique_table_row_condition_with_cnc (GdaConnection *cnc, GdaSqlState
 }
 
 /**
- * gda_compute_unique_table_row_condition
+ * gda_compute_unique_table_row_condition:
  * @stsel: a #GdaSqlSelectStatement
  * @mtable: a #GdaMetaTable
  * @require_pk: set to TRUE if a primary key ir required
@@ -962,19 +1086,18 @@ gda_compute_unique_table_row_condition (GdaSqlStatementSelect *stsel, GdaMetaTab
 }
 
 /**
- * gda_compute_dml_statements
+ * gda_compute_dml_statements:
  * @cnc: a #GdaConnection
  * @select_stmt: a SELECT #GdaStatement (compound statements not handled)
  * @require_pk: TRUE if the created statement have to use a primary key
- * @insert_stmt: a place to store the created INSERT statement, or %NULL
- * @update_stmt: a place to store the created UPDATE statement, or %NULL
- * @delete_stmt: a place to store the created DELETE statement, or %NULL
- * @error: a place to store errors, or %NULL
+ * @insert_stmt: (allow-none) (transfer full): a place to store the created INSERT statement, or %NULL
+ * @update_stmt: (allow-none) (transfer full): a place to store the created UPDATE statement, or %NULL
+ * @delete_stmt: (allow-none) (transfer full): a place to store the created DELETE statement, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * Creates an INSERT, an UPDATE and a DELETE statement from a SELECT statement
- * using the database metadata available in @cnc's meta store.
- *
- * %FALSE will be returned if any of the statement cannot be built.
+ * using the database metadata available in @cnc's meta store. Each statements are computed only if
+ * the corresponding place to store the created statement is not %NULL.
  * 
  * returns: %TRUE if no error occurred
  */
@@ -1121,6 +1244,12 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			pspec->g_type = tcol->gtype != G_TYPE_INVALID ? tcol->gtype: G_TYPE_STRING;
 			pspec->nullok = tcol->nullok;
 			expr = gda_sql_expr_new (GDA_SQL_ANY_PART (ist));
+			if (tcol->default_value)
+				g_value_set_string ((expr->value = gda_value_new (G_TYPE_STRING)),
+						    tcol->default_value);
+			else if (gda_meta_table_column_get_attribute (tcol, GDA_ATTRIBUTE_AUTO_INCREMENT))
+				expr->value = gda_value_new_default (GDA_EXTRA_AUTO_INCREMENT);
+
 			expr->param_spec = pspec;
 			insert_values_list = g_slist_append (insert_values_list, expr);
 		}
@@ -1130,6 +1259,11 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 			pspec->g_type = tcol->gtype != G_TYPE_INVALID ? tcol->gtype: G_TYPE_STRING;
 			pspec->nullok = tcol->nullok;
 			expr = gda_sql_expr_new (GDA_SQL_ANY_PART (ust));
+			if (tcol->default_value)
+				g_value_set_string ((expr->value = gda_value_new (G_TYPE_STRING)),
+						    tcol->default_value);
+			else if (gda_meta_table_column_get_attribute (tcol, GDA_ATTRIBUTE_AUTO_INCREMENT))
+				expr->value = gda_value_new_default (GDA_EXTRA_AUTO_INCREMENT);
 			expr->param_spec = pspec;
 			ust->expr_list = g_slist_append (ust->expr_list, expr);
 		}
@@ -1175,12 +1309,12 @@ gda_compute_dml_statements (GdaConnection *cnc, GdaStatement *select_stmt, gbool
 }
 
 /**
- * gda_compute_select_statement_from_update
+ * gda_compute_select_statement_from_update:
  * @update_stmt: an UPDATE statement
  * @error: a place to store errors, or %NULL
  *
  * Computes a SELECT statement which selects all the rows the @update_stmt would update. Beware
- * however that this GdaSqlStatement does not select anything (ie it would be rendered as "SELECT FROM ... WHERE ...")
+ * however that this #GdaSqlStatement does not select anything (ie it would be rendered as "SELECT FROM ... WHERE ...")
  * and before being usable, one needs to add some fields to actually select.
  *
  * Returns: a new #GdaStatement if no error occurred, or %NULL otherwise
@@ -1229,13 +1363,287 @@ gda_compute_select_statement_from_update (GdaStatement *update_stmt, GError **er
 	return sel_stmt;
 }
 
+typedef struct  {
+	GdaSqlAnyPart *contents;
+	GdaSet *params;
+	GSList *expr_list; /* contains a list of #GdaSqlExpr after
+			    * gda_sql_any_part_foreach has been called */
+} NullData;
+
+static gboolean
+null_param_foreach_func (GdaSqlAnyPart *part, NullData *data , GError **error)
+{
+	if ((part->type != GDA_SQL_ANY_EXPR) || !((GdaSqlExpr*) part)->param_spec)
+		return TRUE;
+
+	if (!part->parent ||
+	    (part->parent->type != GDA_SQL_ANY_SQL_OPERATION) ||
+	    ((((GdaSqlOperation*) part->parent)->operator_type != GDA_SQL_OPERATOR_TYPE_EQ) &&
+	     (((GdaSqlOperation*) part->parent)->operator_type != GDA_SQL_OPERATOR_TYPE_DIFF)))
+		return TRUE;
+
+	GdaHolder *holder;
+	GdaSqlParamSpec *pspec = ((GdaSqlExpr*) part)->param_spec;
+	holder = gda_set_get_holder (data->params, pspec->name);
+	if (!holder)
+		return TRUE;
+	
+	const GValue *cvalue;
+	cvalue = gda_holder_get_value (holder);
+	if (!cvalue || (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL))
+		return TRUE;
+
+	GdaSqlOperation *op;
+	GdaSqlExpr *oexpr = NULL;
+	op = (GdaSqlOperation*) part->parent;
+	if (op->operands->data == part) {
+		if (op->operands->next)
+			oexpr = (GdaSqlExpr*) op->operands->next->data;
+	}
+	else
+		oexpr = (GdaSqlExpr*) op->operands->data;
+	if (oexpr && !g_slist_find (data->expr_list, oexpr)) /* handle situations where ##p1==##p2
+							      * and both p1 and p2 are set to NULL */
+		data->expr_list = g_slist_prepend (data->expr_list, part);
+	return TRUE;
+}
+
+static GdaSqlExpr *
+get_prev_expr (GSList *expr_list, GdaSqlAnyPart *expr)
+{
+	GSList *list;
+	for (list = expr_list; list && list->next; list = list->next) {
+		if ((GdaSqlAnyPart*) list->next->data == expr)
+			return (GdaSqlExpr*) list->data;
+	}
+	return NULL;
+}
+
+static gboolean
+null_param_unknown_foreach_func (GdaSqlAnyPart *part, NullData *data, GError **error)
+{
+	GdaSqlExpr *expr;
+	if ((part->type != GDA_SQL_ANY_EXPR) || !((GdaSqlExpr*) part)->param_spec)
+		return TRUE;
+
+	if (!part->parent || part->parent != data->contents)
+		return TRUE;
+
+	GdaHolder *holder;
+	GdaSqlParamSpec *pspec = ((GdaSqlExpr*) part)->param_spec;
+	holder = gda_set_get_holder (data->params, pspec->name);
+	if (!holder)
+		return TRUE;
+	
+	const GValue *cvalue;
+	cvalue = gda_holder_get_value (holder);
+	if (!cvalue || (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL))
+		return TRUE;
+
+	GSList *tmplist = NULL;
+	for (expr = get_prev_expr (((GdaSqlStatementUnknown*) data->contents)->expressions, part);
+	     expr;
+	     expr = get_prev_expr (((GdaSqlStatementUnknown*) data->contents)->expressions, (GdaSqlAnyPart*) expr)) {
+		gchar *str, *tmp;
+		if (!expr->value || (G_VALUE_TYPE (expr->value) != G_TYPE_STRING))
+			goto out;
+
+		str = (gchar*) g_value_get_string (expr->value);
+		if (!str || !*str) {
+			tmplist = g_slist_prepend (tmplist, expr);
+			continue;
+		}
+		for (tmp = str + strlen (str) - 1; tmp >= str; tmp --) {
+			if ((*tmp == ' ') || (*tmp == '\t') || (*tmp == '\n') || (*tmp == '\r'))
+				continue;
+			if (*tmp == '=') {
+				gchar *dup;
+				if ((tmp > str) && (*(tmp-1) == '!')) {
+					*(tmp-1) = 0;
+					dup = g_strdup_printf ("%s IS NOT NULL", str);
+				}
+				else {
+					*tmp = 0;
+					dup = g_strdup_printf ("%s IS NULL", str);
+				}
+				g_value_take_string (expr->value, dup);
+				if (tmplist) {
+					data->expr_list = g_slist_concat (tmplist, data->expr_list);
+					tmplist = NULL;
+				}
+				data->expr_list = g_slist_prepend (data->expr_list, part);
+				goto out;
+			}
+			else
+				goto out;
+		}
+		tmplist = g_slist_prepend (tmplist, expr);
+	}
+ out:
+	g_slist_free (tmplist);
+
+	return TRUE;
+}
+
+/**
+ * gda_rewrite_sql_statement_for_null_parameters: (skip):
+ * @sqlst: (transfer full): a #GdaSqlStatement
+ * @params: a #GdaSet to be used as parameters when executing @stmt
+ * @out_modified: (allow-none): a place to store the boolean which tells if @stmt has been modified or not, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Modifies @sqlst to take into account any parameter which might be %NULL: if @sqlst contains the
+ * equivalent of "xxx = &lt;parameter definition&gt;" and if that parameter is in @params and
+ * its value is of type GDA_TYPE_NUL, then that part is replaced with "xxx IS NULL". It also
+ * handles the "xxx IS NOT NULL" transformation.
+ *
+ * If @out_modified is not %NULL, then it will be set to %TRUE if @sqlst has been modified
+ * by this function, and to %FALSE otherwise.
+ *
+ * This function is used by provider's implementations to make sure one can use parameters with
+ * NULL values in statements without having to rewrite statements, as database usually don't
+ * consider that "xxx = NULL" is the same as "xxx IS NULL" when using parameters.
+ *
+ * Returns: (transfer full): the modified @sqlst statement, or %NULL if an error occurred
+ *
+ * Since: 4.2.9
+ */
+GdaSqlStatement *
+gda_rewrite_sql_statement_for_null_parameters (GdaSqlStatement *sqlst, GdaSet *params,
+					       gboolean *out_modified, GError **error)
+{
+	if (out_modified)
+		*out_modified = FALSE;
+	g_return_val_if_fail (sqlst, sqlst);
+
+	if (!params)
+		return sqlst;
+	GSList *list;
+	for (list = params->holders; list; list = list->next) {
+		const GValue *cvalue;
+		cvalue = gda_holder_get_value ((GdaHolder*) list->data);
+		if (cvalue && (G_VALUE_TYPE (cvalue) == GDA_TYPE_NULL))
+			break;
+	}
+	if (!list || (sqlst->stmt_type == GDA_SQL_STATEMENT_NONE))
+		return sqlst; /* no modifications necessary */
+
+	NullData data;
+	data.contents = GDA_SQL_ANY_PART (sqlst->contents);
+	data.params = params;
+	data.expr_list = NULL;
+
+	if (sqlst->stmt_type == GDA_SQL_STATEMENT_UNKNOWN) {
+		if (! gda_sql_any_part_foreach (GDA_SQL_ANY_PART (sqlst->contents),
+						(GdaSqlForeachFunc) null_param_unknown_foreach_func,
+						&data, error)) {
+			gda_sql_statement_free (sqlst);
+			return NULL;
+		}
+		if (out_modified)
+			*out_modified = data.expr_list ? TRUE : FALSE;
+		for (list = data.expr_list; list; list = list->next) {
+			((GdaSqlStatementUnknown*) data.contents)->expressions = 
+				g_slist_remove (((GdaSqlStatementUnknown*) data.contents)->expressions,
+						list->data);
+			gda_sql_expr_free ((GdaSqlExpr*) list->data);
+		}
+	}
+	else {
+		if (! gda_sql_any_part_foreach (GDA_SQL_ANY_PART (sqlst->contents),
+						(GdaSqlForeachFunc) null_param_foreach_func,
+						&data, error)) {
+			gda_sql_statement_free (sqlst);
+			return NULL;
+		}
+		if (out_modified)
+			*out_modified = data.expr_list ? TRUE : FALSE;
+		for (list = data.expr_list; list; list = list->next) {
+			GdaSqlOperation *op;
+			op = (GdaSqlOperation*) (((GdaSqlAnyPart*) list->data)->parent);
+			op->operands = g_slist_remove (op->operands, list->data);
+			if (op->operator_type == GDA_SQL_OPERATOR_TYPE_EQ)
+				op->operator_type = GDA_SQL_OPERATOR_TYPE_ISNULL;
+			else
+				op->operator_type = GDA_SQL_OPERATOR_TYPE_ISNOTNULL;
+			gda_sql_expr_free ((GdaSqlExpr*) list->data);
+		}
+	}
+	g_slist_free (data.expr_list);
+	return sqlst;
+}
+
+/**
+ * gda_rewrite_statement_for_null_parameters:
+ * @stmt: (transfer none): a #GdaStatement
+ * @params: a #GdaSet to be used as parameters when executing @stmt
+ * @out_stmt: (transfer full) (allow-none): a place to store the new #GdaStatement, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Modifies @stmt to take into account any parameter which might be %NULL: if @stmt contains the
+ * equivalent of "xxx = &lt;parameter definition&gt;" and if that parameter is in @params and
+ * its value is of type GDA_TYPE_NUL, then that part is replaced with "xxx IS NULL". It also
+ * handles the "xxx IS NOT NULL" transformation.
+ *
+ * For example the following SELECT:
+ * <programlisting>SELECT * FROM data WHERE id = ##id::int::null AND name = ##name::string</programlisting>
+ * in case the "id" parameter is set to NULL, is converted to:
+ * <programlisting>SELECT * FROM data WHERE id IS NULL AND name = ##name::string</programlisting>
+ *
+ * if @out_stmt is not %NULL, then it will contain:
+ * <itemizedlist>
+ *   <listitem><para>the modified statement if some modifications were required and no error occured (the function returns %TRUE)</para></listitem>
+ *   <listitem><para>%NULL if no modification to @stmt were required and no erro occurred (the function returns %FALSE)</para></listitem>
+ *   <listitem><para>%NULL if an error occured (the function returns %TRUE)</para></listitem>
+ * </itemizedlist>
+ *
+ * This function is used by provider's implementations to make sure one can use parameters with
+ * NULL values in statements without having to rewrite statements, as database usually don't
+ * consider that "xxx = NULL" is the same as "xxx IS NULL" when using parameters.
+ *
+ * Returns: %TRUE if @stmt needs to be transformed to handle NULL parameters, and %FALSE otherwise
+ *
+ * Since: 4.2.9
+ */
+gboolean
+gda_rewrite_statement_for_null_parameters (GdaStatement *stmt, GdaSet *params,
+					   GdaStatement **out_stmt, GError **error)
+{
+	GdaSqlStatement *sqlst;
+	gboolean mod;
+	g_return_val_if_fail (GDA_IS_STATEMENT (stmt), FALSE);
+	g_return_val_if_fail (!params || GDA_IS_SET (params), FALSE);
+
+	if (out_stmt)
+		*out_stmt = NULL;
+	if (!params)
+		return FALSE;
+
+	g_object_get ((GObject*) stmt, "structure", &sqlst, NULL);
+	if (gda_rewrite_sql_statement_for_null_parameters (sqlst, params, &mod, error)) {
+		if (out_stmt) {
+			if (mod) {
+				*out_stmt = g_object_new (GDA_TYPE_STATEMENT, "structure", sqlst, NULL);
+				gda_sql_statement_free (sqlst);
+			}
+		}
+		return mod;
+	}
+	else {
+		/* error => leave *out_stmt to %NULL */
+		return TRUE;
+	}
+}
+
+
+
 static gboolean stmt_rewrite_insert_remove (GdaSqlStatementInsert *ins, GdaSet *params, GError **error);
 static gboolean stmt_rewrite_insert_default_keyword (GdaSqlStatementInsert *ins, GdaSet *params, GError **error);
 static gboolean stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd, GdaSet *params, GError **error);
 
 
 /**
- * gda_statement_rewrite_for_default_values
+ * gda_statement_rewrite_for_default_values:
  * @stmt: a #GdaStatement object
  * @params: a #GdaSet containing the variable's values to be bound when executing @stmt
  * @remove: set to %TRUE if DEFAULT fields are removed, of %FALSE if the "DEFAULT" keyword is used
@@ -1245,7 +1653,7 @@ static gboolean stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd,
  * (as returned by gda_holder_value_is_default()) are either removed from the statement (if @remove
  * is %TRUE) or replaced by the "DEFAULT" keyword (if @remove is %FALSE).
  *
- * This function is only usefull for database providers' implementations which have to deal with default
+ * This function is only useful for database providers' implementations which have to deal with default
  * values when executing statements, and is only relevant in the case of INSERT or UPDATE statements
  * (in the latter case an error is returned if @remove is %TRUE).
  *
@@ -1459,7 +1867,7 @@ stmt_rewrite_update_default_keyword (GdaSqlStatementUpdate *upd, GdaSet *params,
 }
 
 /**
- * gda_identifier_hash
+ * gda_identifier_hash:
  * @id: an identifier string
  *
  * computes a hash string from @id, to be used in hash tables as a #GHashFunc
@@ -1491,7 +1899,7 @@ gda_identifier_hash (const gchar *id)
 }
 
 /**
- * gda_identifier_equal
+ * gda_identifier_equal:
  * @id1: an identifier string
  * @id2: an identifier string
  *
@@ -1897,10 +2305,10 @@ gda_sql_identifier_split (const gchar *id)
 static gboolean _sql_identifier_needs_quotes (const gchar *str);
 
 /**
- * gda_sql_identifier_quote
+ * gda_sql_identifier_quote:
  * @id: an SQL identifier
- * @cnc: a #GdaConnection object, or %NULL
- * @prov: a #GdaServerProvider object, or %NULL
+ * @cnc: (allow-none): a #GdaConnection object, or %NULL
+ * @prov: (allow-none): a #GdaServerProvider object, or %NULL
  * @for_meta_store set to %TRUE if the returned string will be used in a #GdaMetaStore
  * @force_quotes: set to %TRUE to force the returned string to be quoted
  *
@@ -2221,7 +2629,7 @@ static char rfc1738_reserved_chars[] =
 };
 
 /**
- * gda_rfc1738_encode
+ * gda_rfc1738_encode:
  * @string: a string to encode 
  *
  * Encodes @string using the RFC 1738 recommendations: the
@@ -2291,7 +2699,7 @@ gda_rfc1738_encode (const gchar *string)
 }
 
 /**
- * gda_rfc1738_decode
+ * gda_rfc1738_decode:
  * @string: a string to decode
  *
  * Decodes @string using the RFC 1738 recommendations: the
@@ -2359,7 +2767,7 @@ gda_rfc1738_decode (gchar *string)
 
 
 /**
- * gda_dsn_split
+ * gda_dsn_split:
  * @string: a string in the "[&lt;username&gt;[:&lt;password&gt;]@]&lt;DSN&gt;" form
  * @out_dsn: a place to store the new string containing the &lt;DSN&gt; part
  * @out_username: a place to store the new string containing the &lt;username&gt; part
@@ -2407,7 +2815,7 @@ gda_dsn_split (const gchar *string, gchar **out_dsn, gchar **out_username, gchar
 }
 
 /**
- * gda_connection_string_split
+ * gda_connection_string_split:
  * @string: a string in the "[&lt;provider&gt;://][&lt;username&gt;[:&lt;password&gt;]@]&lt;connection_params&gt;" form
  * @out_cnc_params: a place to store the new string containing the &lt;connection_params&gt; part
  * @out_provider: a place to store the new string containing the &lt;provider&gt; part
@@ -2593,7 +3001,7 @@ _parse_iso8601_date (GDate *gdate, const gchar *value, char **out_endptr)
 }
 
 /**
- * gda_parse_iso8601_date
+ * gda_parse_iso8601_date:
  * @gdate: a pointer to a #GDate structure which will be filled
  * @value: a string
  *
@@ -2690,7 +3098,7 @@ _parse_iso8601_time (GdaTime *timegda, const gchar *value, char **out_endptr)
 }
 
 /**
- * gda_parse_iso8601_time
+ * gda_parse_iso8601_time:
  * @timegda: a pointer to a #GdaTime structure which will be filled
  * @value: a string
  *
@@ -2717,7 +3125,7 @@ gda_parse_iso8601_time (GdaTime *timegda, const gchar *value)
 }
 
 /**
- * gda_parse_iso8601_timestamp
+ * gda_parse_iso8601_timestamp:
  * @timestamp: a pointer to a #GdaTimeStamp structure which will be filled
  * @value: a string
  *
