@@ -1,25 +1,29 @@
-/* gda-handler-time.c
+/*
+ * Copyright (C) 2006 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
- * Copyright (C) 2003 - 2009 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gda-handler-time.h"
 #include <string.h>
+#include <ctype.h>
 #include <glib/gi18n-lib.h>
 
 static void gda_handler_time_class_init (GdaHandlerTimeClass *class);
@@ -93,7 +97,8 @@ gda_handler_time_get_type (void)
 			NULL,
 			sizeof (GdaHandlerTime),
 			0,
-			(GInstanceInitFunc) gda_handler_time_init
+			(GInstanceInitFunc) gda_handler_time_init,
+			0
 		};		
 
 		static const GInterfaceInfo data_entry_info = {
@@ -194,11 +199,11 @@ gda_handler_time_dispose (GObject *object)
 }
 
 /**
- * gda_handler_time_new
+ * gda_handler_time_new:
  *
  * Creates a data handler for time values
  *
- * Returns: the new object
+ * Returns: (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_time_new (void)
@@ -212,12 +217,12 @@ gda_handler_time_new (void)
 }
 
 /**
- * gda_handler_time_new_no_locale
+ * gda_handler_time_new_no_locale:
  *
  * Creates a data handler for time values, but using the default C locale
  * instead of the current user locale.
  *
- * Returns: the new object
+ * Returns: (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_time_new_no_locale (void)
@@ -230,7 +235,7 @@ gda_handler_time_new_no_locale (void)
 }
 
 /**
- * gda_handler_time_set_sql_spec
+ * gda_handler_time_set_sql_spec:
  * @dh: a #GdaHandlerTime object
  * @first: what comes first in the date representation
  * @sec: what comes second in the date representation
@@ -262,6 +267,41 @@ gda_handler_time_set_sql_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec
 	dh->priv->sql_locale->separator = separator;
 }
 
+/**
+ * gda_handler_time_set_str_spec:
+ * @dh: a #GdaHandlerTime object
+ * @first: what comes first in the date representation
+ * @sec: what comes second in the date representation
+ * @third: what comes third in the date representation
+ * @separator: separator character used between year, month and day
+ * @twodigits_years: TRUE if year part of date must be rendered on 2 digits
+ *
+ * Specifies the human readable output style of the @dh data handler.
+ * The general format is "FIRSTsSECsTHIRD"
+ * where FIRST, SEC and THIRD are specified by @first, @sec and @trird and 's' is the separator,
+ * specified by @separator.
+ *
+ * The default implementation depends on the current locale, except if @dh was created
+ * using gda_handler_time_new_no_locale().
+ *
+ * Since: 4.2.1
+ */
+void
+gda_handler_time_set_str_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec,
+				GDateDMY third, gchar separator, gboolean twodigits_years)
+{
+	g_return_if_fail (GDA_IS_HANDLER_TIME (dh));
+	g_return_if_fail (dh->priv);
+	g_return_if_fail (first != sec);
+	g_return_if_fail (sec != third);
+	g_return_if_fail (first != third);
+
+	dh->priv->str_locale->dmy_order[0] = first;
+	dh->priv->str_locale->dmy_order[1] = sec;
+	dh->priv->str_locale->dmy_order[2] = third;
+	dh->priv->str_locale->twodigit_years = twodigits_years;
+	dh->priv->str_locale->separator = separator;
+}
 
 static void
 handler_compute_locale (GdaHandlerTime *hdl)
@@ -283,9 +323,10 @@ handler_compute_locale (GdaHandlerTime *hdl)
 	if (*ptr) {
 		hdl->priv->str_locale->separator = *ptr;
 		*ptr = 0;
-		nums[0] = atoi (numstart);
+		nums[0] = atoi (numstart); /* Flawfinder: ignore */
 	}
-	else error = TRUE;
+	else
+		error = TRUE;
 
 	/* 2nd number */
 	if (!error) {
@@ -295,9 +336,10 @@ handler_compute_locale (GdaHandlerTime *hdl)
 			ptr++;
 		if (*ptr) {
 			*ptr = 0;
-			nums[1] = atoi (numstart);
+			nums[1] = atoi (numstart); /* Flawfinder: ignore */
 		}
-		else error = TRUE;
+		else
+			error = TRUE;
 	}
 
 	/* 3rd number */
@@ -307,7 +349,7 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		while (*ptr && g_ascii_isdigit (*ptr))
 			ptr++;
 		*ptr = 0;
-		nums[2] = atoi (numstart);
+		nums[2] = atoi (numstart); /* Flawfinder: ignore */
 	}
 	
 	/* computations */
@@ -319,7 +361,7 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		time_t now;
 		struct tm *now_tm;
 
-		for (i=0; i<3; i++) {
+		for (i=0; i < 3; i++) {
 			switch (nums[i]) {
 			case 7:
 				hdl->priv->str_locale->dmy_order[i] = G_DATE_MONTH;
@@ -338,7 +380,16 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		}
 		
 		now = time (NULL);
-		now_tm = localtime (&now);
+#ifdef HAVE_LOCALTIME_R
+		struct tm tmpstm;
+		now_tm = localtime_r (&now, &tmpstm);
+#elif HAVE_LOCALTIME_S
+		struct tm tmpstm;
+		g_assert (localtime_s (&tmpstm, &now) == 0);
+		now_tm = &tmpstm;
+#else
+                now_tm = localtime (&now);
+#endif
 		hdl->priv->str_locale->current_offset = ((now_tm->tm_year + 1900) / 100) * 100;
 
 #ifdef GDA_DEBUG_NO		
@@ -373,7 +424,7 @@ handler_compute_locale (GdaHandlerTime *hdl)
 }
 
 /**
- * gda_handler_time_get_no_locale_str_from_value
+ * gda_handler_time_get_no_locale_str_from_value:
  * @hdl: a #GdaHandlerTime object
  * @value: a GValue value
  *
@@ -446,7 +497,7 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *hdl, const GValue
 }
 
 /**
- * gda_handler_time_get_format
+ * gda_handler_time_get_format:
  * @dh: a #GdaHandlerTime object
  * @type: the type of data being handled
  *
@@ -741,7 +792,8 @@ gda_handler_time_get_value_from_str (GdaDataHandler *iface, const gchar *sql, GT
 
 static gboolean make_timestamp (GdaHandlerTime *hdl, GdaTimestamp *timestamp, 
 				const gchar *value, LocaleSetting *locale);
-static gboolean make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *locale);
+static gboolean make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value,
+			   LocaleSetting *locale, const gchar **out_endptr);
 static gboolean make_time (GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value);
 static GValue *
 gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql, 
@@ -759,7 +811,7 @@ gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql,
 	g_return_val_if_fail (hdl->priv, NULL);
 
 	if (type == G_TYPE_DATE) {
-		if (make_date (hdl, &date, sql, locale)) {
+		if (make_date (hdl, &date, sql, locale, NULL)) {
 			value = g_value_init (g_new0 (GValue, 1), G_TYPE_DATE);
 			g_value_set_boxed (value, (gconstpointer) &date);
 		}
@@ -790,47 +842,69 @@ static gboolean
 make_timestamp (GdaHandlerTime *hdl, GdaTimestamp *timestamp, const gchar *value, LocaleSetting *locale)
 {
 	gboolean retval;
-	gchar *str, *ptr;
+	const gchar *end_ptr;
 	GDate vdate;
 	GdaTime vtime;
-	char *buff;
+	memset (&vtime, 0, sizeof (GdaTime));
+	vtime.timezone = GDA_TIMEZONE_INVALID;
+
+	retval = make_date (hdl, &vdate, value, locale, &end_ptr);
+	timestamp->day = vdate.day;
+	timestamp->month = vdate.month;
+	timestamp->year = vdate.year;
 
-	str = g_strdup (value);
-	ptr = strtok_r (str, " ", &buff);
-	retval = make_date (hdl, &vdate, ptr, locale);
 	if (retval) {
-		ptr = strtok_r (NULL, " ", &buff);
-		retval = make_time (hdl, &vtime, ptr);
-		if (retval) {
-			timestamp->day = vdate.day;
-			timestamp->month = vdate.month;
-			timestamp->year = vdate.year;
-
-			timestamp->hour = vtime.hour;
-			timestamp->minute = vtime.minute;
-			timestamp->second = vtime.second;
-			timestamp->fraction = vtime.fraction;
-			timestamp->timezone = vtime.timezone;
-		}
+		if (*end_ptr != ' ')
+			retval = FALSE;
+		else
+			retval = make_time (hdl, &vtime, end_ptr + 1);
 	}
-	g_free (str);
+
+	timestamp->hour = vtime.hour;
+	timestamp->minute = vtime.minute;
+	timestamp->second = vtime.second;
+	timestamp->fraction = vtime.fraction;
+	timestamp->timezone = vtime.timezone;
 
 	/*g_print ("Value #%s# => %d\n", value, retval);*/
 
 	return retval;
 }
 
-
-/* Makes a GDate from a string like "24-12-2003" */
 static gboolean
-make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *locale)
+get_uint_from_string (const gchar *str, guint16 *out_int)
+{
+	long int li;
+	char *endptr = NULL;
+	li = strtol (str, &endptr, 10);
+	if (!*endptr && (li >= 0) && (li <= G_MAXUINT16)) {
+		*out_int = (guint16) li;
+		return TRUE;
+	}
+	else {
+		*out_int = 0;
+		return FALSE;
+	}
+}
+
+/* Makes a GDate from a string like "24-12-2003"
+ * If @out_endptr is %NULL, then all the @value has to be consumed and there must not
+ * be any character left. If it's not %NULL, then it will point on the first unused character
+ * of @value.
+ */
+static gboolean
+make_date (G_GNUC_UNUSED GdaHandlerTime *hdl, GDate *date, const gchar *value,
+	   LocaleSetting *locale, const gchar **out_endptr)
 {
 	gboolean retval = TRUE;
-	gushort nums[3];
+	guint16 nums[3];
 	gboolean error = FALSE;
 	gchar *ptr, *numstart, *tofree;
 	gint i;
 
+	if (out_endptr)
+		*out_endptr = NULL;
+
 	if (!value)
 		return FALSE;
 
@@ -845,47 +919,70 @@ make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *
 		ptr++;
 	if ((ptr != numstart) && *ptr) {
 		*ptr = 0;
-		nums[0] = atoi (numstart);
+		if (! get_uint_from_string (numstart, &(nums[0])))
+			error = TRUE;
 	}
-	else error = TRUE;
+	else
+		error = TRUE;
 
 	/* 2nd number */
 	if (!error) {
-		ptr++;
-		numstart = ptr;
-		while (*ptr && g_ascii_isdigit (*ptr))
+		if (value [ptr-tofree] != locale->separator)
+			error = TRUE;
+		else {
 			ptr++;
-		if ((ptr != numstart) && *ptr) {
-			*ptr = 0;
-			nums[1] = atoi (numstart);
+			numstart = ptr;
+			while (*ptr && g_ascii_isdigit (*ptr))
+				ptr++;
+			if ((ptr != numstart) && *ptr) {
+				*ptr = 0;
+				if (! get_uint_from_string (numstart, &(nums[1])))
+					error = TRUE;
+			}
+			else
+				error = TRUE;
 		}
-		else error = TRUE;
 	}
 
 	/* 3rd number */
 	if (!error) {
-		ptr++;
-		numstart = ptr;
-		while (*ptr && g_ascii_isdigit (*ptr))
+		if (value [ptr-tofree] != locale->separator)
+			error = TRUE;
+		else {
 			ptr++;
-		*ptr = 0;
-		if (ptr != numstart)
-			nums[2] = atoi (numstart);
-		else
+			numstart = ptr;
+			while (*ptr && g_ascii_isdigit (*ptr))
+				ptr++;
+			*ptr = 0;
+			if (ptr != numstart) {
+				if (! get_uint_from_string (numstart, &(nums[2])))
+					error = TRUE;
+			}
+			else
+				error = TRUE;
+		}
+	}
+
+	/* test if there are some characters left */
+	if (!error) {
+		if (out_endptr)
+			*out_endptr = value + (ptr-tofree);
+		else if (value [ptr-tofree])
 			error = TRUE;
 	}
 
+	/* analyse what's parsed */
 	if (!error) {
 		for (i=0; i<3; i++) {
 			switch (locale->dmy_order[i]) {
 			case G_DATE_DAY:
-				if (g_date_valid_day (nums[i]))
+				if ((nums[i] <= G_MAXUINT8) && g_date_valid_day ((GDateDay) nums[i]))
 					g_date_set_day (date, nums[i]);
 				else
 					retval = FALSE;
 				break;
 			case G_DATE_MONTH:
-				if (g_date_valid_month (nums[i]))
+				if ((nums[i] <= 12) && g_date_valid_month ((GDateMonth) nums[i]))
 					g_date_set_month (date, nums[i]);
 				else
 					retval = FALSE;
@@ -923,105 +1020,99 @@ make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value, LocaleSetting *
  * Also works if there is only 0 or 1 digit instead of 2
  */
 static gboolean
-make_time (GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value)
+make_time (G_GNUC_UNUSED GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value)
 {
 	const gchar *ptr;
 
+	memset (timegda, 0, sizeof (GdaTime));
+	timegda->timezone = GDA_TIMEZONE_INVALID;
+
 	if (!value)
 		return FALSE;
 
 	/* hour */
-	timegda->fraction = 0;
-	timegda->timezone = GDA_TIMEZONE_INVALID;
 	ptr = value;
-	if ((*ptr >= '0') && (*ptr <= '9') &&
-	    (*(ptr+1) >= '0') && (*(ptr+1) <= '9')) {
-		timegda->hour = (*ptr - '0') * 10 + *(ptr+1) - '0';
-		ptr += 2;
-	}
-	else if ((*ptr >= '0') && (*ptr <= '9') && (ptr[1] == ':')) {
-		timegda->hour = *ptr - '0';
+	if (!isdigit (*ptr))
+		return FALSE;
+
+	timegda->hour = *ptr - '0';
+	ptr++;
+	if (isdigit (*ptr)) {
+		timegda->hour = timegda->hour * 10 + (*ptr - '0');
 		ptr++;
 	}
-	else if (*ptr == ':')
-		timegda->hour = 0;
-	else
+	if (timegda->hour >= 24)
+		return FALSE;
+
+	if (*ptr == ':')
+		ptr++;
+	else if (!isdigit (*ptr))
 		return FALSE;
 
 	/* minute */
-	if (! *ptr)
+	timegda->minute = *ptr - '0';
+	ptr++;
+	if (isdigit (*ptr)) {
+		timegda->minute = timegda->minute * 10 + (*ptr - '0');
+		ptr++;
+	}
+	if (timegda->minute >= 60)
 		return FALSE;
 	if (*ptr == ':')
 		ptr++;
-	if ((*ptr >= '0') && (*ptr <= '9') &&
-	    (*(ptr+1) >= '0') && (*(ptr+1) <= '9')) {
-		timegda->minute = (*ptr - '0') * 10 + *(ptr+1) - '0';
-		ptr += 2;
-	}
-	else if ((*ptr >= '0') && (*ptr <= '9') && (ptr[1] == ':')) {
-		timegda->minute = *ptr - '0';
-		ptr++;
-	}
-	else if (*ptr == ':')
-		timegda->minute = 0;
-	else
+	else if (!isdigit (*ptr))
 		return FALSE;
 
 	/* second */
-	timegda->second = 0;
-	if (! *ptr) {
-		if ((timegda->hour > 24) || (timegda->minute > 60))
-			return FALSE;
-		else
-			return TRUE;
-	}
-	if (*ptr == ':')
-		ptr++;
-	if ((*ptr >= '0') && (*ptr <= '9') &&
-	    (*(ptr+1) >= '0') && (*(ptr+1) <= '9')) {
-		timegda->second = (*ptr - '0') * 10 + *(ptr+1) - '0';
-		ptr += 2;
-	}
-	else if ((*ptr >= '0') && (*ptr <= '9')) {
-		timegda->second = *ptr - '0';
+	timegda->second = *ptr - '0';
+	ptr++;
+	if (isdigit (*ptr)) {
+		timegda->second = timegda->second * 10 + (*ptr - '0');
 		ptr++;
 	}
-	else if (*ptr == ':')
-		timegda->second = 0;
-	
-	/* extra */
-	if (! *ptr) {
-		if ((timegda->hour > 24) || (timegda->minute > 60) || 
-		    (timegda->second > 60))
-			return FALSE;
-		else
-			return TRUE;
-	}
-
-	if (*ptr == '.') {
-		ptr ++;
-		while (*ptr && (*ptr >= '0') && (*ptr <= '9')) {
-			timegda->fraction = timegda->fraction * 10 + *ptr - '0';
-			ptr++;
-		}
-	}
-
-	if ((*ptr == '+') || (*ptr == '-')) {
-		glong sign = (*ptr == '+') ? 1 : -1;
-		ptr ++;
-		timegda->timezone = 0;
-		while (*ptr && (*ptr >= '0') && (*ptr <= '9')) {
-			timegda->timezone = timegda->timezone * 10 + sign * ((*ptr) - '0');
-			ptr++;
-		}
-		timegda->timezone *= 3600;
-	}
-	
-	/* checks */
-	if ((timegda->hour > 24) || (timegda->minute > 60) || (timegda->second > 60))
+	if (timegda->second >= 60)
 		return FALSE;
-	else
-		return TRUE;
+
+	/* fraction */
+	if (*ptr && (*ptr != '.') && (*ptr != '+') && (*ptr != '-'))
+		return FALSE;
+	if (*ptr == '.') {
+		ptr++;
+		if (!isdigit (*ptr))
+			return FALSE;
+		while (isdigit (*ptr)) {
+			unsigned long long lu;
+			lu = timegda->fraction * 10 + (*ptr - '0');
+			if (lu < G_MAXULONG)
+				timegda->fraction = lu;
+			else
+				return FALSE;
+			ptr++;
+		}
+	}
+	
+	/* timezone */
+	if ((*ptr == '-') || (*ptr == '+')) {
+		gint sign = (*ptr == '+') ? 1 : -1;
+		ptr++;
+		if (!isdigit (*ptr))
+			return FALSE;
+		timegda->timezone = sign * (*ptr - '0');
+		ptr++;
+		if (isdigit (*ptr)) {
+			timegda->timezone = timegda->minute * 10 + sign * (*ptr - '0');
+			ptr++;
+		}
+		if (*ptr)
+			return FALSE;
+		if ((timegda->timezone >= -24) && (timegda->timezone <= 24))
+			timegda->timezone *= 60*60;
+		else {
+			timegda->timezone = 0;
+			return FALSE;
+		}
+	}
+	return TRUE;
 }
 
 
@@ -1039,7 +1130,16 @@ gda_handler_time_get_sane_init_value (GdaDataHandler *iface, GType type)
 	g_return_val_if_fail (hdl->priv, NULL);
 
 	now = time (NULL);
+#ifdef HAVE_LOCALTIME_R
+	struct tm tmpstm;
+	stm = localtime_r (&now, &tmpstm);
+#elif HAVE_LOCALTIME_S
+	struct tm tmpstm;
+	g_assert (localtime_s (&tmpstm, &now) == 0);
+	stm = &tmpstm;
+#else
 	stm = localtime (&now);
+#endif
 
 	if (type == G_TYPE_DATE) {
 		GDate *gdate;
