@@ -1,20 +1,22 @@
 /*
- * Copyright (C) 2007 - 2008 Vivien Malerba
+ * Copyright (C) 2008 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2008 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2010 Murray Cumming <murrayc@murrayc.com>
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #ifndef _GDA_STATEMENT_STRUCT_PARTS_H
@@ -47,13 +49,13 @@ typedef struct _GdaSqlSelectOrder GdaSqlSelectOrder;
 /**
  * GdaSqlExpr:
  * @any: inheritance structure
- * @value: a #GValue, or %NULL. Please see specific note about this field. 
- * @param_spec: a #GdaSqlParamSpec, or %NULL if this is not a variable
- * @func: not %NULL if expression is a function or aggregate
- * @cond: not %NULL if expression is a condition or an operation
- * @select: not %NULL if expression is a sub select statement (#GdaSqlStatementSelect or #GdaSqlStatementCompound)
- * @case_s: not %NULL if expression is a CASE WHEN ... expression
- * @cast_as: not %NULL if expression must be cast to another data type
+ * @value: (allow-none): a #GValue, or %NULL. Please see specific note about this field. 
+ * @param_spec: (allow-none): a #GdaSqlParamSpec, or %NULL if this is not a variable
+ * @func: (allow-none): not %NULL if expression is a function or aggregate
+ * @cond: (allow-none): not %NULL if expression is a condition or an operation
+ * @select: (allow-none): not %NULL if expression is a sub select statement (#GdaSqlStatementSelect or #GdaSqlStatementCompound)
+ * @case_s: (allow-none): not %NULL if expression is a CASE WHEN ... expression
+ * @cast_as: (allow-none): not %NULL if expression must be cast to another data type
  * @value_is_ident: Please see specific note about the @value field
  *
  * This structure contains any expression, either as a value (the @value part is set),
@@ -177,6 +179,40 @@ void             gda_sql_function_take_args_list (GdaSqlFunction *function, GSLi
 /*
  * An operation on one or more expressions
  */
+/**
+ * GdaSqlOperatorType:
+ * @GDA_SQL_OPERATOR_TYPE_AND: 
+ * @GDA_SQL_OPERATOR_TYPE_OR: 
+ * @GDA_SQL_OPERATOR_TYPE_EQ: 
+ * @GDA_SQL_OPERATOR_TYPE_IS: 
+ * @GDA_SQL_OPERATOR_TYPE_LIKE:
+ * @GDA_SQL_OPERATOR_TYPE_ILIKE:  
+ * @GDA_SQL_OPERATOR_TYPE_BETWEEN: 
+ * @GDA_SQL_OPERATOR_TYPE_GT: 
+ * @GDA_SQL_OPERATOR_TYPE_LT: 
+ * @GDA_SQL_OPERATOR_TYPE_GEQ: 
+ * @GDA_SQL_OPERATOR_TYPE_LEQ: 
+ * @GDA_SQL_OPERATOR_TYPE_DIFF: 
+ * @GDA_SQL_OPERATOR_TYPE_REGEXP: 
+ * @GDA_SQL_OPERATOR_TYPE_REGEXP_CI: 
+ * @GDA_SQL_OPERATOR_TYPE_NOT_REGEXP: 
+ * @GDA_SQL_OPERATOR_TYPE_NOT_REGEXP_CI: 
+ * @GDA_SQL_OPERATOR_TYPE_SIMILAR: 
+ * @GDA_SQL_OPERATOR_TYPE_ISNULL: 
+ * @GDA_SQL_OPERATOR_TYPE_ISNOTNULL: 
+ * @GDA_SQL_OPERATOR_TYPE_NOT: 
+ * @GDA_SQL_OPERATOR_TYPE_IN: 
+ * @GDA_SQL_OPERATOR_TYPE_NOTIN: 
+ * @GDA_SQL_OPERATOR_TYPE_CONCAT: 
+ * @GDA_SQL_OPERATOR_TYPE_PLUS: 
+ * @GDA_SQL_OPERATOR_TYPE_MINUS: 
+ * @GDA_SQL_OPERATOR_TYPE_STAR: 
+ * @GDA_SQL_OPERATOR_TYPE_DIV: 
+ * @GDA_SQL_OPERATOR_TYPE_REM: 
+ * @GDA_SQL_OPERATOR_TYPE_BITAND: 
+ * @GDA_SQL_OPERATOR_TYPE_BITOR: 
+ * @GDA_SQL_OPERATOR_TYPE_BITNOT:
+ */
 typedef enum {
 	GDA_SQL_OPERATOR_TYPE_AND,
 	GDA_SQL_OPERATOR_TYPE_OR,
@@ -209,7 +245,8 @@ typedef enum {
 	GDA_SQL_OPERATOR_TYPE_REM,
 	GDA_SQL_OPERATOR_TYPE_BITAND,
 	GDA_SQL_OPERATOR_TYPE_BITOR,
-	GDA_SQL_OPERATOR_TYPE_BITNOT
+	GDA_SQL_OPERATOR_TYPE_BITNOT,
+	GDA_SQL_OPERATOR_TYPE_ILIKE
 } GdaSqlOperatorType;
 
 struct _GdaSqlOperation {
@@ -323,6 +360,17 @@ typedef enum {
 	GDA_SQL_SELECT_JOIN_RIGHT,
 	GDA_SQL_SELECT_JOIN_FULL
 } GdaSqlSelectJoinType;
+
+/**
+ * GdaSqlSelectJoin:
+ * @any: inheritance structure
+ * @type: type of join
+ * @position: represents a join between a target at (pos &lt; @position) and the one at @position
+ * @expr: (allow-none): joining expression, or %NULL
+ * @use: (allow-none): list of #GdaSqlField pointers to use when joining, or %NULL
+ *
+ * This structure represents a join between two targets in a SELECT statement.
+ */
 struct _GdaSqlSelectJoin
 {
 	GdaSqlAnyPart         any;
@@ -348,6 +396,14 @@ const gchar       *gda_sql_select_join_type_to_string (GdaSqlSelectJoinType type
 /*
  * Any FROM ... in a SELECT statement
  */
+/**
+ * GdaSqlSelectFrom:
+ * @any: inheritance structure
+ * @targets: (element-type Gda.SqlSelectTarget): list of #GdaSqlSelectTarget
+ * @joins: (element-type Gda.SqlSelectJoin): list of #GdaSqlSelectJoin
+ *
+ * This structure represents the FROM clause of a SELECT statement, it lists targets and joins
+ */
 struct _GdaSqlSelectFrom
 {
 	GdaSqlAnyPart    any;
