@@ -1,24 +1,21 @@
-/* Gda Sqlite provider
- * Copyright (C) 2008 - 2010 The GNOME Foundation
+/*
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *         Vivien Malerba <malerba@gnome-db.org>
- *         Bas Driessen <bas.driessen@xobas.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -45,7 +42,8 @@ typedef enum {
 	I_PRAGMA_INDEX_LIST,
 	I_PRAGMA_INDEX_INFO,
 	I_PRAGMA_FK_LIST,
-	I_PRAGMA_PROCLIST
+	I_PRAGMA_PROCLIST,
+	I_PRAGMA_FK_ENFORCED
 } InternalStatementItem;
 
 
@@ -69,7 +67,10 @@ static gchar *internal_sql[] = {
 	"PRAGMA foreign_key_list (##tblname::string)",
 
 	/* I_PRAGMA_PROCLIST */
-	"PRAGMA proc_list"
+	"PRAGMA proc_list",
+
+	/* I_PRAGMA_FK_ENFORCED */
+	"PRAGMA foreign_keys"
 };
 /* name of the temporary database we don't want */
 #define TMP_DATABASE_NAME "temp"
@@ -93,7 +94,8 @@ static GValue       *view_check_option;
 static GValue       *false_value;
 static GValue       *true_value;
 static GValue       *zero_value;
-static GValue       *rule_value;
+static GValue       *rule_value_none;
+static GValue       *rule_value_action;
 static GdaSet       *pragma_set;
 
 static GValue *
@@ -175,7 +177,8 @@ _gda_sqlite_provider_meta_init (GdaServerProvider *provider)
 	g_value_set_boolean ((false_value = gda_value_new (G_TYPE_BOOLEAN)), FALSE);
 	g_value_set_boolean ((true_value = gda_value_new (G_TYPE_BOOLEAN)), TRUE);
 	g_value_set_int ((zero_value = gda_value_new (G_TYPE_INT)), 0);
-	g_value_set_string ((rule_value = gda_value_new (G_TYPE_STRING)), "NONE");
+	rule_value_none = view_check_option;
+	g_value_set_string ((rule_value_action = gda_value_new (G_TYPE_STRING)), "NO ACTION");
 
 	pragma_set = gda_set_new_inline (2, "tblname", G_TYPE_STRING, "",
 					 "idxname", G_TYPE_STRING, "");
@@ -236,7 +239,7 @@ get_statement (InternalStatementItem type, const gchar *schema_name, const gchar
 }
 
 gboolean
-_gda_sqlite_meta__info (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__info (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
 			GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	GdaDataModel *model;
@@ -255,12 +258,12 @@ _gda_sqlite_meta__info (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__btypes (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__btypes (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
 			  GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	GdaDataModel *mod_model;
 	gboolean retval = TRUE;
-	gint i;
+	gsize i;
 	typedef struct {
 		gchar *tname;
 		gchar *gtype;
@@ -371,7 +374,7 @@ fill_udt_model (SqliteConnectionData *cdata, GHashTable *added_hash,
 	if ((status != SQLITE_OK) || !tables_stmt)
 		return FALSE;
 	
-	if (!cdata->types)
+	if (!cdata->types_hash)
                 _gda_sqlite_compute_types_hash (cdata);
 
 	for (status = SQLITE3_CALL (sqlite3_step) (tables_stmt);
@@ -397,11 +400,13 @@ fill_udt_model (SqliteConnectionData *cdata, GHashTable *added_hash,
 		     fields_status == SQLITE_ROW; 
 		     fields_status = SQLITE3_CALL (sqlite3_step) (fields_stmt)) {
 			GType gtype;
+			GType *pg;
 			const gchar *typname = (gchar *) SQLITE3_CALL (sqlite3_column_text) (fields_stmt, 2);
 			if (!typname || !(*typname)) 
 				continue;
 
-			gtype = GPOINTER_TO_INT (g_hash_table_lookup (cdata->types, typname));
+			pg = g_hash_table_lookup (cdata->types_hash, typname);
+			gtype = pg ? *pg : GDA_TYPE_NULL;
 			if ((gtype == GDA_TYPE_NULL) && !g_hash_table_lookup (added_hash, typname)) {
 				GValue *vname, *vgtyp;
 
@@ -450,7 +455,7 @@ nocase_str_equal (gconstpointer v1, gconstpointer v2)
 }
 
 gboolean
-_gda_sqlite_meta__udt (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__udt (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 		       GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	SqliteConnectionData *cdata;
@@ -506,9 +511,9 @@ _gda_sqlite_meta__udt (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta_udt (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_udt (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 		      GdaMetaStore *store, GdaMetaContext *context, GError **error,
-		      const GValue *udt_catalog, const GValue *udt_schema)
+		      G_GNUC_UNUSED const GValue *udt_catalog, const GValue *udt_schema)
 {
 	SqliteConnectionData *cdata;
 	GdaDataModel *mod_model = NULL;
@@ -542,94 +547,104 @@ _gda_sqlite_meta_udt (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__udt_cols (GdaServerProvider *prov, GdaConnection *cnc, 
-			    GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__udt_cols (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			    G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			    G_GNUC_UNUSED GError **error)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_udt_cols (GdaServerProvider *prov, GdaConnection *cnc, 
-			   GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			   const GValue *udt_catalog, const GValue *udt_schema, const GValue *udt_name)
+_gda_sqlite_meta_udt_cols (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			   G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			   G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *udt_catalog,
+			   G_GNUC_UNUSED const GValue *udt_schema, G_GNUC_UNUSED const GValue *udt_name)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__enums (GdaServerProvider *prov, GdaConnection *cnc, 
-			 GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__enums (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			 G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			 G_GNUC_UNUSED GError **error)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_enums (GdaServerProvider *prov, GdaConnection *cnc, 
-			GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			const GValue *udt_catalog, const GValue *udt_schema, const GValue *udt_name)
+_gda_sqlite_meta_enums (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *udt_catalog,
+			G_GNUC_UNUSED const GValue *udt_schema, G_GNUC_UNUSED const GValue *udt_name)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__domains (GdaServerProvider *prov, GdaConnection *cnc, 
-			   GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__domains (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			   G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			   G_GNUC_UNUSED GError **error)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_domains (GdaServerProvider *prov, GdaConnection *cnc, 
-			  GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			  const GValue *domain_catalog, const GValue *domain_schema)
+_gda_sqlite_meta_domains (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			  G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			  G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *domain_catalog,
+			  G_GNUC_UNUSED const GValue *domain_schema)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__constraints_dom (GdaServerProvider *prov, GdaConnection *cnc, 
-				   GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__constraints_dom (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+				   G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+				   G_GNUC_UNUSED GError **error)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_constraints_dom (GdaServerProvider *prov, GdaConnection *cnc, 
-				  GdaMetaStore *store, GdaMetaContext *context, GError **error,
-				  const GValue *domain_catalog, const GValue *domain_schema, 
-				  const GValue *domain_name)
+_gda_sqlite_meta_constraints_dom (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+				  G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+				  G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *domain_catalog,
+				  G_GNUC_UNUSED const GValue *domain_schema,
+				  G_GNUC_UNUSED const GValue *domain_name)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__el_types (GdaServerProvider *prov, GdaConnection *cnc, 
-			    GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__el_types (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			    G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			    G_GNUC_UNUSED GError **error)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_el_types (GdaServerProvider *prov, GdaConnection *cnc, 
-			   GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			   const GValue *specific_name)
+_gda_sqlite_meta_el_types (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			   G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			   G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *specific_name)
 {
 	/* feature not supported by SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__collations (GdaServerProvider *prov, GdaConnection *cnc, 
-			      GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__collations (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			      G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			      G_GNUC_UNUSED GError **error)
 {
 	/* FIXME: We need to do something similar to what's done with
 	 * functions and aggregates as there is no pragma or API */
@@ -637,10 +652,11 @@ _gda_sqlite_meta__collations (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta_collations (GdaServerProvider *prov, GdaConnection *cnc, 
-			     GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			     const GValue *collation_catalog, const GValue *collation_schema, 
-			     const GValue *collation_name_n)
+_gda_sqlite_meta_collations (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			     G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			     G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *collation_catalog,
+			     G_GNUC_UNUSED const GValue *collation_schema,
+			     G_GNUC_UNUSED const GValue *collation_name_n)
 {
 	/* FIXME: We need to do something similar to what's done with
 	 * functions and aggregates as there is no pragma or API */
@@ -648,8 +664,8 @@ _gda_sqlite_meta_collations (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__character_sets (GdaServerProvider *prov, GdaConnection *cnc, 
-				  GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__character_sets (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+				  G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context, G_GNUC_UNUSED GError **error)
 {
 	/* FIXME: We need to do something similar to what's done with
 	 * functions and aggregates as there is no pragma or API */
@@ -657,10 +673,10 @@ _gda_sqlite_meta__character_sets (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta_character_sets (GdaServerProvider *prov, GdaConnection *cnc, 
-				 GdaMetaStore *store, GdaMetaContext *context, GError **error,
-				 const GValue *chset_catalog, const GValue *chset_schema, 
-				 const GValue *chset_name_n)
+_gda_sqlite_meta_character_sets (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+				 G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context, G_GNUC_UNUSED GError **error,
+				 G_GNUC_UNUSED const GValue *chset_catalog, G_GNUC_UNUSED const GValue *chset_schema,
+				 G_GNUC_UNUSED const GValue *chset_name_n)
 {
 	/* FIXME: We need to do something similar to what's done with
 	 * functions and aggregates as there is no pragma or API */
@@ -676,9 +692,9 @@ _gda_sqlite_meta__schemata (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean 
-_gda_sqlite_meta_schemata (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_schemata (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			   GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			   const GValue *catalog_name, const GValue *schema_name_n)
+			   G_GNUC_UNUSED const GValue *catalog_name, const GValue *schema_name_n)
 {
 	GdaDataModel *model, *tmpmodel;
 	gboolean retval = TRUE;
@@ -841,7 +857,7 @@ fill_tables_views_model (GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__tables_views (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__tables_views (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 				GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	GdaDataModel *tmpmodel;
@@ -897,9 +913,9 @@ _gda_sqlite_meta__tables_views (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean 
-_gda_sqlite_meta_tables_views (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_tables_views (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			       GdaMetaStore *store, GdaMetaContext *context, GError **error, 
-			       const GValue *table_catalog, const GValue *table_schema, const GValue *table_name_n)
+			       G_GNUC_UNUSED const GValue *table_catalog, const GValue *table_schema, const GValue *table_name_n)
 {
 	GdaDataModel *tables_model, *views_model;
 	gboolean retval = TRUE;
@@ -1034,10 +1050,12 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 		if (pzDataType) {
 			gchar *tmp = g_strdup (pzDataType);
 			gchar *ptr;
+			GType *pg;
 			for (ptr = tmp; *ptr && (*ptr != '(') && (*ptr != '['); ptr++);
 			if (*ptr)
 				*ptr = 0;
-			gtype = GPOINTER_TO_INT (g_hash_table_lookup (cdata->types, tmp));
+			pg = g_hash_table_lookup (cdata->types_hash, tmp);
+			gtype = pg ? (GType) *pg : GDA_TYPE_NULL;
 			g_free (tmp);
 		}
 		if (gtype == 0) 
@@ -1083,7 +1101,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 }
 
 gboolean
-_gda_sqlite_meta__columns (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			   GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	GdaDataModel *mod_model, *tmpmodel;
@@ -1165,9 +1183,9 @@ _gda_sqlite_meta__columns (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta_columns (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			  GdaMetaStore *store, GdaMetaContext *context, GError **error, 
-			  const GValue *table_catalog, const GValue *table_schema, const GValue *table_name)
+			  G_GNUC_UNUSED const GValue *table_catalog, const GValue *table_schema, const GValue *table_name)
 {
 	gboolean retval = TRUE;
 	GdaDataModel *mod_model = NULL;
@@ -1191,18 +1209,18 @@ _gda_sqlite_meta_columns (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__view_cols (GdaServerProvider *prov, GdaConnection *cnc, 
-			     GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__view_cols (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			     G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context, G_GNUC_UNUSED GError **error)
 {
 	/* FIXME: feature not natively supported by SQLite => parse view's definition ? */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_view_cols (GdaServerProvider *prov, GdaConnection *cnc, 
-			    GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			    const GValue *view_catalog, const GValue *view_schema, 
-			    const GValue *view_name)
+_gda_sqlite_meta_view_cols (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			    G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			    G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *view_catalog,
+			    G_GNUC_UNUSED const GValue *view_schema, G_GNUC_UNUSED const GValue *view_name)
 {
 	/* FIXME: feature not natively supported by SQLite => parse view's definition ? */
 	return TRUE;	
@@ -1428,7 +1446,7 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 }
 
 gboolean
-_gda_sqlite_meta__constraints_tab (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__constraints_tab (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 				   GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	GdaDataModel *mod_model, *tmpmodel;
@@ -1509,10 +1527,10 @@ _gda_sqlite_meta__constraints_tab (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean 
-_gda_sqlite_meta_constraints_tab (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_constraints_tab (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 				  GdaMetaStore *store, GdaMetaContext *context, GError **error,
-				  const GValue *table_catalog, const GValue *table_schema, const GValue *table_name,
-				  const GValue *constraint_name_n)
+				  G_GNUC_UNUSED const GValue *table_catalog, const GValue *table_schema,
+				  const GValue *table_name, const GValue *constraint_name_n)
 {
 	gboolean retval = TRUE;
 	GdaDataModel *mod_model = NULL;
@@ -1536,9 +1554,9 @@ _gda_sqlite_meta_constraints_tab (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 static gboolean 
-fill_constraints_ref_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaDataModel *mod_model, 
-			    const GValue *p_table_schema, const GValue *p_table_name, 
-			    const GValue *constraint_name_n, GError **error)
+fill_constraints_ref_model (GdaConnection *cnc, G_GNUC_UNUSED SqliteConnectionData *cdata,
+			    GdaDataModel *mod_model, const GValue *p_table_schema, const GValue *p_table_name,
+			    const GValue *constraint_name_n, gint fk_enforced, GError **error)
 {
 	GdaDataModel *tmpmodel;
 	gboolean retval = TRUE;
@@ -1606,8 +1624,8 @@ fill_constraints_ref_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 					    TRUE, v3, /* ref_table_name */
 					    TRUE, v4, /* ref_constraint_name */
 					    FALSE, NULL, /* match_option */
-					    FALSE, rule_value, /* update_rule */
-					    FALSE, rule_value /* delete_rule */))
+					    FALSE, fk_enforced ? rule_value_action : rule_value_none, /* update_rule */
+					    FALSE, fk_enforced ? rule_value_action : rule_value_none /* delete_rule */))
 				retval = FALSE;
 			if (constraint_name_n)
 				g_free (constname);
@@ -1619,13 +1637,14 @@ fill_constraints_ref_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 }
 
 gboolean
-_gda_sqlite_meta__constraints_ref (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__constraints_ref (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 				   GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	GdaDataModel *mod_model, *tmpmodel;
 	gboolean retval = TRUE;
 	gint i, nrows;
 	SqliteConnectionData *cdata;
+	gint fk_enforced = -1;
 	
 	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
 	if (!cdata)
@@ -1673,13 +1692,31 @@ _gda_sqlite_meta__constraints_ref (GdaServerProvider *prov, GdaConnection *cnc,
 		gint tnrows, ti;
 		tnrows = gda_data_model_get_n_rows (tables_model);
 		for (ti = 0; ti < tnrows; ti++) {
+		       
 			/* iterate through the tables */
 			const GValue *cv = gda_data_model_get_value_at (tables_model, 0, ti, error);
 			if (!cv) {
 				retval = FALSE;
 				break;
 			}
-			if (!fill_constraints_ref_model (cnc, cdata, mod_model, cvalue, cv, NULL, error)) {
+
+ 			if (fk_enforced < 0) {
+				GdaDataModel *pmodel;
+				fk_enforced = 0;
+				pmodel = (GdaDataModel *) gda_connection_statement_execute (cnc,
+							   internal_stmt[I_PRAGMA_FK_ENFORCED],
+							   NULL, GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
+							   NULL, NULL);
+				if (pmodel) {
+					const GValue *pv;
+					pv = gda_data_model_get_value_at (pmodel, 0, 0, NULL);
+					if (pv && (G_VALUE_TYPE (pv) == G_TYPE_INT))
+						fk_enforced = g_value_get_int (pv) ? 1 : 0;
+					g_object_unref (pmodel);
+				}
+			}
+			if (!fill_constraints_ref_model (cnc, cdata, mod_model, cvalue, cv, NULL,
+							 fk_enforced, error)) {
 				retval = FALSE;
 				break;
 			}
@@ -1700,9 +1737,9 @@ _gda_sqlite_meta__constraints_ref (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta_constraints_ref (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_constraints_ref (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 				  GdaMetaStore *store, GdaMetaContext *context, GError **error,
-				  const GValue *table_catalog, const GValue *table_schema, const GValue *table_name, 
+				  G_GNUC_UNUSED const GValue *table_catalog, const GValue *table_schema, const GValue *table_name,
 				  const GValue *constraint_name)
 {
 	gboolean retval = TRUE;
@@ -1716,7 +1753,22 @@ _gda_sqlite_meta_constraints_ref (GdaServerProvider *prov, GdaConnection *cnc,
 	mod_model = gda_meta_store_create_modify_data_model (store, context->table_name);
 	g_assert (mod_model);
 
-	retval = fill_constraints_ref_model (cnc, cdata, mod_model, table_schema, table_name, constraint_name, error);
+	GdaDataModel *pmodel;
+	gint fk_enforced = 0;
+	pmodel = (GdaDataModel *) gda_connection_statement_execute (cnc,
+								    internal_stmt[I_PRAGMA_FK_ENFORCED],
+								    NULL, GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
+								    NULL, NULL);
+	if (pmodel) {
+		const GValue *pv;
+		pv = gda_data_model_get_value_at (pmodel, 0, 0, NULL);
+		if (pv && (G_VALUE_TYPE (pv) == G_TYPE_INT))
+			fk_enforced = g_value_get_int (pv) ? 1 : 0;
+		g_object_unref (pmodel);
+	}
+
+	retval = fill_constraints_ref_model (cnc, cdata, mod_model, table_schema, table_name, constraint_name,
+					     fk_enforced, error);
 	if (retval) {
 		gda_meta_store_set_reserved_keywords_func (store, _gda_sqlite_get_reserved_keyword_func());
 		retval = gda_meta_store_modify_with_context (store, context, mod_model, error);
@@ -1919,7 +1971,7 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 }
 
 gboolean
-_gda_sqlite_meta__key_columns (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta__key_columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			       GdaMetaStore *store, GdaMetaContext *context, GError **error)
 {
 	/* iterate through the tables, and each time call fill_constraints_tab_model() 
@@ -2039,9 +2091,9 @@ _gda_sqlite_meta__key_columns (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean 
-_gda_sqlite_meta_key_columns (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_key_columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			      GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			      const GValue *table_catalog, const GValue *table_schema, const GValue *table_name, 
+			      G_GNUC_UNUSED const GValue *table_catalog, const GValue *table_schema, const GValue *table_name,
 			      const GValue *constraint_name)
 {
 	gboolean retval = TRUE;
@@ -2066,36 +2118,39 @@ _gda_sqlite_meta_key_columns (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__check_columns (GdaServerProvider *prov, GdaConnection *cnc, 
-				 GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__check_columns (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+				 G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context, G_GNUC_UNUSED GError **error)
 {
 	/* FIXME: How to get this ? */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_check_columns (GdaServerProvider *prov, GdaConnection *cnc, 
-				GdaMetaStore *store, GdaMetaContext *context, GError **error,
-				const GValue *table_catalog, const GValue *table_schema, 
-				const GValue *table_name, const GValue *constraint_name)
+_gda_sqlite_meta_check_columns (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+				G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+				G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *table_catalog,
+				G_GNUC_UNUSED const GValue *table_schema,
+				G_GNUC_UNUSED const GValue *table_name,
+				G_GNUC_UNUSED const GValue *constraint_name)
 {
 	/* FIXME: How to get this ? */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__triggers (GdaServerProvider *prov, GdaConnection *cnc, 
-			    GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__triggers (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			    G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			    G_GNUC_UNUSED GError **error)
 {
 	/* FIXME: How to get this ? */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_triggers (GdaServerProvider *prov, GdaConnection *cnc, 
-			   GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			   const GValue *table_catalog, const GValue *table_schema, 
-			   const GValue *table_name)
+_gda_sqlite_meta_triggers (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			   G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			   G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *table_catalog,
+			   G_GNUC_UNUSED const GValue *table_schema, G_GNUC_UNUSED const GValue *table_name)
 {
 	/* FIXME: How to get this ? */
 	return TRUE;
@@ -2112,7 +2167,8 @@ _gda_sqlite_meta__routines (GdaServerProvider *prov, GdaConnection *cnc,
 #ifndef HAVE_SQLITE
 static gboolean
 fill_routines (GdaDataModel *mod_model, 
-	       const GValue *rname, const GValue *is_agg, const GValue *rnargs, const GValue *sname, GError **error)
+	       const GValue *rname, const GValue *is_agg, const GValue *rnargs, const GValue *sname,
+	       GError **error)
 {
 	GValue *v0;
 	gboolean retval = TRUE;
@@ -2152,10 +2208,10 @@ fill_routines (GdaDataModel *mod_model,
 #endif
 
 gboolean
-_gda_sqlite_meta_routines (GdaServerProvider *prov, GdaConnection *cnc, 
+_gda_sqlite_meta_routines (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 			   GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			   const GValue *routine_catalog, const GValue *routine_schema, 
-			   const GValue *routine_name_n)
+			   G_GNUC_UNUSED const GValue *routine_catalog,
+			   G_GNUC_UNUSED const GValue *routine_schema, const GValue *routine_name_n)
 {
 	gboolean retval = TRUE;
 	
@@ -2220,72 +2276,79 @@ _gda_sqlite_meta_routines (GdaServerProvider *prov, GdaConnection *cnc,
 }
 
 gboolean
-_gda_sqlite_meta__routine_col (GdaServerProvider *prov, GdaConnection *cnc, 
-			       GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__routine_col (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			       G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			       G_GNUC_UNUSED GError **error)
 {
 	/* feature not available in SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_routine_col (GdaServerProvider *prov, GdaConnection *cnc, 
-			      GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			      const GValue *rout_catalog, const GValue *rout_schema, 
-			      const GValue *rout_name)
+_gda_sqlite_meta_routine_col (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			      G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			      G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *rout_catalog,
+			      G_GNUC_UNUSED const GValue *rout_schema, G_GNUC_UNUSED const GValue *rout_name)
 {
 	/* feature not available in SQLite */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__routine_par (GdaServerProvider *prov, GdaConnection *cnc, 
-			       GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__routine_par (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			       G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			       G_GNUC_UNUSED GError **error)
 {
 	/* Routines in SQLite accept any type of value => nothing to do */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_routine_par (GdaServerProvider *prov, GdaConnection *cnc, 
-			      GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			      const GValue *rout_catalog, const GValue *rout_schema, 
-			      const GValue *rout_name)
+_gda_sqlite_meta_routine_par (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			      G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			      G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *rout_catalog,
+			      G_GNUC_UNUSED const GValue *rout_schema, G_GNUC_UNUSED const GValue *rout_name)
 {
 	/* Routines in SQLite accept any type of value => nothing to do */
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__indexes_tab (GdaServerProvider *prov, GdaConnection *cnc, 
-			       GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__indexes_tab (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			       G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			       G_GNUC_UNUSED GError **error)
 {
 	//TO_IMPLEMENT;
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_indexes_tab (GdaServerProvider *prov, GdaConnection *cnc, 
-			      GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			      const GValue *table_catalog, const GValue *table_schema, const GValue *table_name,
-			      const GValue *index_name_n)
+_gda_sqlite_meta_indexes_tab (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			      G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			      G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *table_catalog,
+			      G_GNUC_UNUSED const GValue *table_schema,
+			      G_GNUC_UNUSED const GValue *table_name,
+			      G_GNUC_UNUSED const GValue *index_name_n)
 {
 	//TO_IMPLEMENT;
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta__index_cols (GdaServerProvider *prov, GdaConnection *cnc, 
-			      GdaMetaStore *store, GdaMetaContext *context, GError **error)
+_gda_sqlite_meta__index_cols (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			      G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			      G_GNUC_UNUSED GError **error)
 {
 	//TO_IMPLEMENT;
 	return TRUE;
 }
 
 gboolean
-_gda_sqlite_meta_index_cols (GdaServerProvider *prov, GdaConnection *cnc, 
-			     GdaMetaStore *store, GdaMetaContext *context, GError **error,
-			     const GValue *table_catalog, const GValue *table_schema,
-			     const GValue *table_name, const GValue *index_name)
+_gda_sqlite_meta_index_cols (G_GNUC_UNUSED GdaServerProvider *prov, G_GNUC_UNUSED GdaConnection *cnc,
+			     G_GNUC_UNUSED GdaMetaStore *store, G_GNUC_UNUSED GdaMetaContext *context,
+			     G_GNUC_UNUSED GError **error, G_GNUC_UNUSED const GValue *table_catalog,
+			     G_GNUC_UNUSED const GValue *table_schema, G_GNUC_UNUSED const GValue *table_name,
+			     G_GNUC_UNUSED const GValue *index_name)
 {
 	//TO_IMPLEMENT;
 	return TRUE;
