@@ -1,23 +1,23 @@
-/* 
- * Copyright (C) 2007 - 2011 The GNOME Foundation.
+/*
+ * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -131,7 +131,7 @@ gda_vconnection_hub_get_type (void)
  * gda_vconnection_hub_add:
  * @hub: a #GdaVconnectionHub connection
  * @cnc: a #GdaConnection
- * @ns: a namespace, or %NULL
+ * @ns: (allow-none): a namespace, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * Make all the tables of @cnc appear as tables (of the same name) in the @hub connection.
@@ -240,7 +240,7 @@ get_hub_cnc_by_cnc (GdaVconnectionHub *hub, GdaConnection *cnc)
 /**
  * gda_vconnection_hub_get_connection:
  * @hub: a #GdaVconnectionHub connection
- * @ns: a name space, or %NULL
+ * @ns: (allow-none): a name space, or %NULL
  *
  * Find the #GdaConnection object in @hub associated to the @ns name space
  *
@@ -800,15 +800,18 @@ attach_hub_connection (GdaVconnectionHub *hub, HubConnection *hc, GError **error
 	gchar *tmp;
 	GdaMetaStore *store;
 	GdaMetaContext context;
+	GdaConnectionOptions options;
 	
 	store = gda_connection_get_meta_store (hc->cnc);
 	g_assert (store);
-
-	/* make sure the meta store is up to date */
-	context.table_name = "_tables";
-	context.size = 0;
-	if (!gda_connection_update_meta_store (hc->cnc, &context, error))
-		return FALSE;
+	g_object_get ((GObject*) hc->cnc, "options", &options, NULL);
+	if (! (options & GDA_CONNECTION_OPTIONS_AUTO_META_DATA)) {
+		/* make sure the meta store is up to date */
+		context.table_name = "_tables";
+		context.size = 0;
+		if (!gda_connection_update_meta_store (hc->cnc, &context, error))
+			return FALSE;
+	}
 
 	/* add a :memory: database */
 	if (hc->ns) {
