@@ -1,23 +1,22 @@
-/* 
- * Copyright (C) 2007 - 2011 The GNOME Foundation.
+/*
+ * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -245,6 +244,48 @@ static sqlite3_module Module = {
 	virtualRename                 /* Rename - Notification that the table will be given a new name */
 };
 
+/*
+ * handle data model exceptions and return appropriate code
+ */
+static int
+handle_data_model_exception (sqlite3_vtab *pVtab, GdaDataModel *model)
+{
+	GError **exceptions;
+	gint i;
+	exceptions = gda_data_model_get_exceptions (model);
+	if (!exceptions)
+		return SQLITE_OK;
+
+	GError *trunc_error = NULL;
+	GError *fatal_error = NULL;
+	for (i = 0; exceptions [i]; i++) {
+		GError *e;
+		e = exceptions [i];
+		if ((e->domain == GDA_DATA_MODEL_ERROR) &&
+		    (e->code == GDA_DATA_MODEL_TRUNCATED_ERROR))
+			trunc_error = e;
+		else {
+			fatal_error = e;
+			break;
+		}
+	}
+	if (fatal_error || trunc_error) {
+		GError *e;
+		e = fatal_error;
+		if (!e)
+			e = trunc_error;
+		if (pVtab->zErrMsg)
+			SQLITE3_CALL (sqlite3_free) (pVtab->zErrMsg);
+		pVtab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
+			(e->message ? e->message : _("No detail"));
+		if (fatal_error)
+			return SQLITE_ERROR;
+		else
+			return SQLITE_IOERR_TRUNCATE;
+	}
+	return SQLITE_OK;
+}
+
 static GdaConnection *
 gda_vprovider_data_model_create_connection (GdaServerProvider *provider)
 {
@@ -273,10 +314,10 @@ gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnec
 
 	if (params) {
 		m_params = gda_quark_list_copy (params);
-		gda_quark_list_add_from_string (m_params, "_IS_VIRTUAL=TRUE;LOAD_GDA_FUNCTIONS=TRUE", TRUE);
+		gda_quark_list_add_from_string (m_params, "_IS_VIRTUAL=TRUE;EXTRA_FUNCTIONS=TRUE", TRUE);
 	}
 	else
-		m_params = gda_quark_list_new_from_string ("_IS_VIRTUAL=TRUE;LOAD_GDA_FUNCTIONS=TRUE");
+		m_params = gda_quark_list_new_from_string ("_IS_VIRTUAL=TRUE;EXTRA_FUNCTIONS=TRUE");
 
 	if (! GDA_SERVER_PROVIDER_CLASS (parent_class)->open_connection (GDA_SERVER_PROVIDER (provider), cnc, m_params,
 									 auth, NULL, NULL, NULL)) {
@@ -416,7 +457,11 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 
 	td = gda_vconnection_get_table_data_by_unique_name (cnc, spec_name);
 	g_free (spec_name);
-	g_assert (td);
+	if (!td) {
+		/* wrong usage! */
+		*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Wrong usage of Libgda's virtual tables"));
+		return SQLITE_ERROR;
+	}
 
 	/* preparations */
 	if (td->spec->data_model) {
@@ -507,6 +552,8 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 		g_string_append (sql, type);
 		if (! gda_column_get_allow_null (column)) 
 			g_string_append (sql, " NOT NULL");
+		if (gtype == G_TYPE_STRING)
+			g_string_append (sql, " COLLATE LOCALE");
 	}
 
 	/* add a hidden column which contains the row number of the data model */
@@ -567,7 +614,7 @@ virtualDestroy (sqlite3_vtab *pVtab)
 }
 
 static int
-virtualOpen (sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor)
+virtualOpen (G_GNUC_UNUSED sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor)
 {
 	VirtualCursor *cursor;
 
@@ -623,10 +670,8 @@ virtualNext (sqlite3_vtab_cursor *cur)
 	if (!gda_data_model_iter_move_next (cursor->iter)) {
 		if (gda_data_model_iter_is_valid (cursor->iter))
 			return SQLITE_IOERR;
-		else
-			return SQLITE_OK;
 	}
-	return SQLITE_OK;
+	return handle_data_model_exception (cur->pVtab, cursor->model);
 }
 
 static int
@@ -685,6 +730,25 @@ virtualColumn (sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i)
 	}
 }
 
+#if GLIB_CHECK_VERSION (2,22,0)
+#define rowid_equal_func g_int64_equal
+#define rowid_hash_func g_int64_hash
+#else
+/* taken from GLib to ensure compatibility with older GLibe versions */
+static
+gboolean
+rowid_equal_func (gconstpointer v1, gconstpointer v2)
+{
+	return *((const gint64*) v1) == *((const gint64*) v2);
+}
+
+static guint
+rowid_hash_func (gconstpointer v)
+{
+	return (guint) *(const gint64*) v;
+}
+#endif
+
 static int
 virtualRowid (sqlite3_vtab_cursor *cur, sqlite_int64 *pRowid)
 {
@@ -696,7 +760,8 @@ virtualRowid (sqlite3_vtab_cursor *cur, sqlite_int64 *pRowid)
 	*pRowid = gda_data_model_iter_get_row (cursor->iter);
 	if (! vtable->rowid_hash || (vtable->rowid_hash_model == vtable->td->real_model)) {
 		if (! vtable->rowid_hash) {
-			vtable->rowid_hash = g_hash_table_new_full (g_int_hash, g_int_equal,
+			vtable->rowid_hash = g_hash_table_new_full (rowid_hash_func,
+								    rowid_equal_func,
 								    g_free,
 								    (GDestroyNotify) g_object_unref);
 			vtable->rowid_hash_model = vtable->td->real_model;
@@ -864,7 +929,8 @@ virtualFilter (sqlite3_vtab_cursor *pVtabCursor, int idxNum, const char *idxStr,
 
 	cursor->model = g_object_ref (vtable->td->real_model);
 	gda_data_model_iter_move_next (cursor->iter);
-	return SQLITE_OK;
+
+	return handle_data_model_exception (pVtabCursor->pVtab, cursor->model);
 }
 
 #ifdef GDA_DEBUG_VIRTUAL
@@ -1014,7 +1080,6 @@ virtualBestIndex (sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo)
 #endif
 	}
 
-
 	return SQLITE_OK;
 }
 
@@ -1125,16 +1190,14 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 		if (! vtable->td->modif_stmt [ptype]) {
 			if (! stmt) {
 				tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-					(_("No statement provided to modify the data model "
-					   "representing the table"));
+					(_("No statement provided to modify the data"));
 				return SQLITE_READONLY;
 			}
 		
 			GdaSet *params;
 			if (! gda_statement_get_parameters (stmt, &params, NULL) || !params) {
 				tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-					(_("Invalid statement provided to modify the data model "
-					   "representing the table"));
+					(_("Invalid statement provided to modify the data"));
 				g_object_unref (stmt);
 				return SQLITE_READONLY;
 			}
@@ -1153,8 +1216,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 			id = gda_holder_get_id (holder);
 			if (!id) {
 				tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-					(_("Invalid parameter in statement provided to modify "
-					   "the data model representing the table"));
+					(_("Invalid parameter in statement to modify the data"));
 				return SQLITE_READONLY;
 			}
 			if (*id == '+' && id[1]) {
@@ -1207,8 +1269,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 				if (! exec_set) {
 					/* can't give value to param named @id */
 					tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-						(_("Invalid parameter in statement provided to modify "
-						   "the data model representing the table"));
+						(_("Invalid parameter in statement to modify the data"));
 					return SQLITE_READONLY;
 				}
 				eh = gda_set_get_holder (exec_set, id);
@@ -1216,8 +1277,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 				    ! gda_holder_set_bind (holder, eh, NULL)) {
 					/* can't give value to param named @id */
 					tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-						(_("Invalid parameter in statement provided to modify "
-						   "the data model representing the table"));
+						(_("Invalid parameter in statement to modify the data"));
 					return SQLITE_READONLY;
 				}
 			}
@@ -1225,14 +1285,14 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 
 		GdaConnection *cnc;
 		cnc = gda_data_select_get_connection (GDA_DATA_SELECT (vtable->td->real_model));
-		
+
+		GError *lerror = NULL;
 #ifdef GDA_DEBUG_NO
 		gchar *sql;
-		GError *lerror = NULL;
 		sql = gda_statement_to_sql (stmt, NULL, NULL);
 		g_print ("SQL: [%s] ", sql);
 		g_free (sql);
-		sql = gda_statement_to_sql_extended (stmt, cnc, params, GDA_STATEMENT_SQL_PRETTY, NULL, &lerror);
+		sql = gda_statement_to_sql_extended (stmt, cnc, vtable->td->modif_params [ptype], GDA_STATEMENT_SQL_PRETTY, NULL, &lerror);
 		if (sql) {
 			g_print ("With params: [%s]\n", sql);
 			g_free (sql);
@@ -1246,11 +1306,12 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 		if (!cnc ||
 		    (gda_connection_statement_execute_non_select (cnc, stmt,
 								  vtable->td->modif_params [ptype],
-								  NULL, NULL) == -1)) {
+								  NULL, &lerror) == -1)) {
 			/* failed to execute */
 			tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-				(_("Failed to execute the statement provided to modify "
-				   "the data model representing the table"));
+				(_("Failed to modify data: %s"),
+				 lerror && lerror->message ? lerror->message : _("No detail"));
+			g_clear_error (&lerror);
 			return SQLITE_READONLY;
 		}
 		return SQLITE_OK;
@@ -1340,7 +1401,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 }
 
 static int
-virtualBegin (sqlite3_vtab *tab)
+virtualBegin (G_GNUC_UNUSED sqlite3_vtab *tab)
 {
 	TRACE (tab, NULL);
 	/* no documentation currently available, don't do anything */
@@ -1379,7 +1440,7 @@ virtualRollback (G_GNUC_UNUSED sqlite3_vtab *tab)
 }
 
 static int
-virtualRename (sqlite3_vtab *pVtab, G_GNUC_UNUSED const char *zNew)
+virtualRename (G_GNUC_UNUSED sqlite3_vtab *pVtab, G_GNUC_UNUSED const char *zNew)
 {
 	TRACE (pVtab, NULL);
 	/* not yet analysed and implemented */
