@@ -1,24 +1,32 @@
-/* GDA Mysql provider
- * Copyright (C) 2008 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) 2001 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2002 Andrew Hill <andru@src.gnome.org>
+ * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@ximian.com>
+ * Copyright (C) 2002 Holger Thon <holger.thon@gnome-db.org>
+ * Copyright (C) 2002 Zbigniew Chyla <cyba@gnome.pl>
+ * Copyright (C) 2003 Chris Silles <csilles@src.gnome.org>
+ * Copyright (C) 2003 Laurent Sansonetti <lrz@gnome.org>
+ * Copyright (C) 2003 Paisa Seeluangsawat <paisa@users.sf.net>
+ * Copyright (C) 2004 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 Alan Knowles <alan@akbkhome.com>
+ * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2005 Mike Fisk <mfisk@woozle.org>
+ * Copyright (C) 2005 lvaro Pea <alvaropg@telefonica.net>
  *
- * AUTHORS:
- *      Carlos Savoretti <csavoretti@gmail.com>
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdarg.h>
@@ -97,7 +105,7 @@ static GObjectClass *parent_class = NULL;
  */
 static void
 gda_mysql_recordset_init (GdaMysqlRecordset       *recset,
-			  GdaMysqlRecordsetClass  *klass)
+			  G_GNUC_UNUSED GdaMysqlRecordsetClass  *klass)
 {
 	g_return_if_fail (GDA_IS_MYSQL_RECORDSET (recset));
 	recset->priv = g_new0 (GdaMysqlRecordsetPrivate, 1);
@@ -290,7 +298,8 @@ gda_mysql_recordset_get_type (void)
 			NULL,
 			sizeof (GdaMysqlRecordset),
 			0,
-			(GInstanceInitFunc) gda_mysql_recordset_init
+			(GInstanceInitFunc) gda_mysql_recordset_init,
+			NULL
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -302,7 +311,7 @@ gda_mysql_recordset_get_type (void)
 }
 
 static GType
-_gda_mysql_type_to_gda (MysqlConnectionData *cdata,
+_gda_mysql_type_to_gda (G_GNUC_UNUSED MysqlConnectionData *cdata,
 			enum enum_field_types  mysql_type, unsigned int charsetnr)
 {
 	GType gtype = 0;
@@ -704,7 +713,7 @@ gda_mysql_recordset_fetch_nb_rows (GdaDataSelect *model)
 }
 
 static GdaRow *
-new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, gint rownum, GError **error)
+new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, G_GNUC_UNUSED gint rownum, GError **error)
 {
 	//g_print ("%s(): NCOLS=%d  ROWNUM=%d\n", __func__, ((GdaDataSelect *) imodel)->prep_stmt->ncols, rownum);
 	int res;
@@ -765,7 +774,7 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, gint rownum, GError **error)
 		long long longlongvalue = 0;
 		double doublevalue = 0.0;
 		float floatvalue = 0.;
-		MYSQL_TIME timevalue = { 0 };
+		MYSQL_TIME timevalue = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 		my_bool is_null = FALSE;
 		unsigned long length;
 		
@@ -919,14 +928,10 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, gint rownum, GError **error)
 				gda_value_set_binary (value, &binary);
 			}
 			else if (type == GDA_TYPE_BLOB) {
-				/* web don't use GdaMysqlBlobOp because it looks like the MySQL
+				/* we don't use GdaMysqlBlobOp because it looks like the MySQL
 				 * API does not support BLOBs accessed in a random way,
 				 * so we return the whole BLOB at once */
-				GdaBlob blob = {
-					.data.data = (guchar*) strvalue,
-					.data.binary_length = length,
-					.op = NULL
-				};
+				GdaBlob blob = { {(guchar*) strvalue, length}, NULL };
 				gda_value_set_blob (value, &blob);
 			}
 			else if (type == G_TYPE_DOUBLE) {
@@ -1086,10 +1091,10 @@ gda_mysql_recordset_fetch_next (GdaDataSelect  *model,
  * can use gda_data_select_take_row().
  */
 static gboolean 
-gda_mysql_recordset_fetch_prev (GdaDataSelect  *model,
-				GdaRow        **row,
-				gint            rownum,
-				GError        **error)
+gda_mysql_recordset_fetch_prev (G_GNUC_UNUSED GdaDataSelect  *model,
+				G_GNUC_UNUSED GdaRow        **row,
+				G_GNUC_UNUSED gint            rownum,
+				G_GNUC_UNUSED GError        **error)
 {
 	/*GdaMysqlRecordset *imodel = (GdaMysqlRecordset*) model;*/
 
@@ -1111,10 +1116,10 @@ gda_mysql_recordset_fetch_prev (GdaDataSelect  *model,
  * can use gda_data_select_take_row().
  */
 static gboolean 
-gda_mysql_recordset_fetch_at (GdaDataSelect  *model,
-			      GdaRow        **row,
-			      gint            rownum,
-			      GError        **error)
+gda_mysql_recordset_fetch_at (G_GNUC_UNUSED GdaDataSelect  *model,
+			      G_GNUC_UNUSED GdaRow        **row,
+			      G_GNUC_UNUSED gint            rownum,
+			      G_GNUC_UNUSED GError        **error)
 {
 	/*GdaMysqlRecordset *imodel = (GdaMysqlRecordset*) model;*/
 	
