@@ -1,22 +1,24 @@
-/* GDA postgres provider
- * Copyright (C) 2007 - 2009 The GNOME Foundation.
- *
- * AUTHORS:
- *         Vivien Malerba <malerba@gnome-db.org>
+/*
+ * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
+ * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 - 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include "gda-postgres-handler-bin.h"
@@ -44,7 +46,7 @@ static gboolean     gda_postgres_handler_bin_accepts_g_type         (GdaDataHand
 static const gchar *gda_postgres_handler_bin_get_descr              (GdaDataHandler *dh);
 
 struct  _GdaPostgresHandlerBinPriv {
-	gchar             *detailled_descr;
+	gchar             *detailed_descr;
 	guint              nb_g_types;
 	GType             *valid_g_types;
 	GdaConnection     *cnc;
@@ -69,7 +71,8 @@ gda_postgres_handler_bin_get_type (void)
 			NULL,
 			sizeof (GdaPostgresHandlerBin),
 			0,
-			(GInstanceInitFunc) gda_postgres_handler_bin_init
+			(GInstanceInitFunc) gda_postgres_handler_bin_init,
+			0
 		};		
 
 		static const GInterfaceInfo data_entry_info = {
@@ -116,7 +119,7 @@ gda_postgres_handler_bin_init (GdaPostgresHandlerBin * hdl)
 {
 	/* Private structure */
 	hdl->priv = g_new0 (GdaPostgresHandlerBinPriv, 1);
-	hdl->priv->detailled_descr = _("Postgresql binary handler");
+	hdl->priv->detailed_descr = _("Postgresql binary handler");
 	hdl->priv->nb_g_types = 2;
 	hdl->priv->valid_g_types = g_new0 (GType, 1);
 	hdl->priv->valid_g_types[0] = GDA_TYPE_BINARY;
