@@ -1,25 +1,33 @@
-/* GDA postgres provider
- * Copyright (C) 1998 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) 2001 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * Copyright (C) 2001 - 2004 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2002 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2002 Zbigniew Chyla <cyba@gnome.pl>
+ * Copyright (C) 2003 Akira TAGOH <tagoh@gnome-db.org>
+ * Copyright (C) 2004 - 2005 Alan Knowles <alank@src.gnome.org>
+ * Copyright (C) 2004 Denis Loginov <dloginov@crl.nmsu.edu>
+ * Copyright (C) 2004 Jos Mara Casanova Crespo <jmcasanova@igalia.com>
+ * Copyright (C) 2004 Julio M. Merino Vidal <jmmv@menta.net>
+ * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2005 lvaro Pea <alvaropg@telefonica.net>
+ * Copyright (C) 2006 - 2010 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *         Vivien Malerba <malerba@gnome-db.org>
- *         Rodrigo Moya <rodrigo@gnome-db.org>
- *         Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #undef GDA_DISABLE_DEPRECATED
@@ -291,7 +299,7 @@ gda_postgres_provider_class_init (GdaPostgresProviderClass *klass)
 }
 
 static void
-gda_postgres_provider_init (GdaPostgresProvider *postgres_prv, GdaPostgresProviderClass *klass)
+gda_postgres_provider_init (GdaPostgresProvider *postgres_prv, G_GNUC_UNUSED GdaPostgresProviderClass *klass)
 {
 	InternalStatementItem i;
 	GdaSqlParser *parser;
@@ -323,7 +331,8 @@ gda_postgres_provider_get_type (void)
 			NULL, NULL,
 			sizeof (GdaPostgresProvider),
 			0,
-			(GInstanceInitFunc) gda_postgres_provider_init
+			(GInstanceInitFunc) gda_postgres_provider_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -339,7 +348,7 @@ gda_postgres_provider_get_type (void)
  * Get provider name request
  */
 static const gchar *
-gda_postgres_provider_get_name (GdaServerProvider *provider)
+gda_postgres_provider_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return POSTGRES_PROVIDER_NAME;
 }
@@ -348,7 +357,7 @@ gda_postgres_provider_get_name (GdaServerProvider *provider)
  * Get provider's version, no need to change this
  */
 static const gchar *
-gda_postgres_provider_get_version (GdaServerProvider *provider)
+gda_postgres_provider_get_version (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return PACKAGE_VERSION;
 }
@@ -388,7 +397,8 @@ pq_notice_processor (GdaConnection *cnc, const char *message)
 static gboolean
 gda_postgres_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
 				       GdaQuarkList *params, GdaQuarkList *auth,
-				       guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data)
+				       G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb,
+				       G_GNUC_UNUSED gpointer cb_data)
 {
 	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -656,7 +666,7 @@ gda_postgres_provider_get_database (GdaServerProvider *provider, GdaConnection *
  */
 static gboolean
 gda_postgres_provider_supports_operation (GdaServerProvider *provider, GdaConnection *cnc,
-					  GdaServerOperationType type, GdaSet *options)
+					  GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options)
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -696,7 +706,8 @@ gda_postgres_provider_supports_operation (GdaServerProvider *provider, GdaConnec
  */
 static GdaServerOperation *
 gda_postgres_provider_create_operation (GdaServerProvider *provider, GdaConnection *cnc,
-					GdaServerOperationType type, GdaSet *options, GError **error)
+					GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options,
+					GError **error)
 {
         gchar *file;
         GdaServerOperation *op;
@@ -852,8 +863,9 @@ gda_postgres_provider_render_operation (GdaServerProvider *provider, GdaConnecti
  */
 static gboolean
 gda_postgres_provider_perform_operation (GdaServerProvider *provider, GdaConnection *cnc,
-					 GdaServerOperation *op, guint *task_id,
-					 GdaServerProviderAsyncCallback async_cb, gpointer cb_data, GError **error)
+					 GdaServerOperation *op, G_GNUC_UNUSED guint *task_id,
+					 GdaServerProviderAsyncCallback async_cb,
+					 G_GNUC_UNUSED gpointer cb_data, GError **error)
 {
         GdaServerOperationType optype;
 
@@ -1051,7 +1063,7 @@ gda_postgres_provider_begin_transaction (GdaServerProvider *provider, GdaConnect
  */
 static gboolean
 gda_postgres_provider_commit_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-					  const gchar *name, GError **error)
+					  G_GNUC_UNUSED const gchar *name, GError **error)
 {
 	PostgresConnectionData *cdata;
 
@@ -1075,7 +1087,7 @@ gda_postgres_provider_commit_transaction (GdaServerProvider *provider, GdaConnec
 static gboolean
 gda_postgres_provider_rollback_transaction (GdaServerProvider *provider,
 					    GdaConnection *cnc,
-					    const gchar *name, GError **error)
+					    G_GNUC_UNUSED const gchar *name, GError **error)
 {
 	PostgresConnectionData *cdata;
 
@@ -1424,7 +1436,8 @@ gda_postgres_provider_get_default_dbms_type (GdaServerProvider *provider, GdaCon
  * by the database. See the PostgreSQL provider implementation for an example.
  */
 static GdaSqlParser *
-gda_postgres_provider_create_parser (GdaServerProvider *provider, GdaConnection *cnc)
+gda_postgres_provider_create_parser (G_GNUC_UNUSED GdaServerProvider *provider,
+				     G_GNUC_UNUSED GdaConnection *cnc)
 {
 	return (GdaSqlParser*) g_object_new (GDA_TYPE_POSTGRES_PARSER, "tokenizer-flavour",
                                              GDA_SQL_PARSER_FLAVOUR_POSTGRESQL, NULL);
@@ -1926,7 +1939,13 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 			}
 		}
 		if (!h) {
-			if (! allow_noparam) {
+			if (allow_noparam) {
+				/* bind param to NULL */
+				param_values [i] = NULL;
+				empty_rs = TRUE;
+				continue;
+			}
+			else {
 				gchar *str;
 				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
 				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
@@ -1936,15 +1955,15 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 				g_free (str);
 				break;
 			}
-			else {
+		}
+		if (!gda_holder_is_valid (h)) {
+			if (allow_noparam) {
 				/* bind param to NULL */
 				param_values [i] = NULL;
 				empty_rs = TRUE;
 				continue;
 			}
-		}
-		if (!gda_holder_is_valid (h)) {
-			if (! allow_noparam) {
+			else {
 				gchar *str;
 				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
 				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
@@ -1954,12 +1973,6 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 				g_free (str);
 				break;
 			}
-			else {
-				/* bind param to NULL */
-				param_values [i] = NULL;
-				empty_rs = TRUE;
-				continue;
-			}
 		}
 		else if (gda_holder_value_is_default (h) && !gda_holder_get_value (h)) {
 			/* create a new GdaStatement to handle all default values and execute it instead */
@@ -1997,8 +2010,39 @@ gda_postgres_provider_statement_execute (GdaServerProvider *provider, GdaConnect
 
 		/* actual binding using the C API, for parameter at position @i */
 		const GValue *value = gda_holder_get_value (h);
-		if (!value || gda_value_is_null (value))
-			param_values [i] = NULL;
+		if (!value || gda_value_is_null (value)) {
+			GdaStatement *rstmt;
+			if (! gda_rewrite_statement_for_null_parameters (stmt, params, &rstmt, error))
+				param_values [i] = NULL;
+			else if (!rstmt)
+				return NULL;
+			else {
+				GObject *obj;
+				params_freev (param_values, param_mem, nb_params);
+				g_free (param_lengths);
+				g_free (param_formats);
+				if (transaction_started)
+					gda_connection_rollback_transaction (cnc, NULL, NULL);
+
+				obj = gda_postgres_provider_statement_execute (provider, cnc,
+									       rstmt, params,
+									       model_usage,
+									       col_types,
+									       last_inserted_row,
+									       task_id, async_cb,
+									       cb_data, error);
+				g_object_unref (rstmt);
+				if (GDA_IS_DATA_SELECT (obj)) {
+					GdaPStmt *pstmt;
+					g_object_get (obj, "prepared-stmt", &pstmt, NULL);
+					if (pstmt) {
+						gda_pstmt_set_gda_statement (pstmt, stmt);
+						g_object_unref (pstmt);
+					}
+				}
+				return obj;
+			}
+		}
 		else if (G_VALUE_TYPE (value) == GDA_TYPE_BLOB) {
 			/* specific BLOB treatment */
 			GdaBlob *blob = (GdaBlob*) gda_value_get_blob ((GValue *) value);
@@ -2186,7 +2230,7 @@ gda_postgres_provider_xa_start (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_postgres_provider_xa_end (GdaServerProvider *provider, GdaConnection *cnc,
-			      const GdaXaTransactionId *xid, GError **error)
+			      const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
@@ -2455,7 +2499,7 @@ pg_remove_quotes (gchar *str)
 }
 
 static gchar *
-gda_postgresql_identifier_quote (GdaServerProvider *provider, GdaConnection *cnc,
+gda_postgresql_identifier_quote (G_GNUC_UNUSED GdaServerProvider *provider, GdaConnection *cnc,
 				 const gchar *id,
 				 gboolean for_meta_store, gboolean force_quotes)
 {
