@@ -1,8 +1,9 @@
-/* GDA Capi provider
- * Copyright (C) 2008 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) YEAR The GNOME Foundation.
  *
  * AUTHORS:
  *      TO_ADD: your name and email
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -259,7 +260,7 @@ gda_capi_provider_class_init (GdaCapiProviderClass *klass)
 }
 
 static void
-gda_capi_provider_init (GdaCapiProvider *capi_prv, GdaCapiProviderClass *klass)
+gda_capi_provider_init (GdaCapiProvider *capi_prv, G_GNUC_UNUSED GdaCapiProviderClass *klass)
 {
 	InternalStatementItem i;
 	GdaSqlParser *parser;
@@ -293,7 +294,8 @@ gda_capi_provider_get_type (void)
 			NULL, NULL,
 			sizeof (GdaCapiProvider),
 			0,
-			(GInstanceInitFunc) gda_capi_provider_init
+			(GInstanceInitFunc) gda_capi_provider_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -309,7 +311,7 @@ gda_capi_provider_get_type (void)
  * Get provider name request
  */
 static const gchar *
-gda_capi_provider_get_name (GdaServerProvider *provider)
+gda_capi_provider_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return CAPI_PROVIDER_NAME;
 }
@@ -318,7 +320,7 @@ gda_capi_provider_get_name (GdaServerProvider *provider)
  * Get provider's version, no need to change this
  */
 static const gchar *
-gda_capi_provider_get_version (GdaServerProvider *provider)
+gda_capi_provider_get_version (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return PACKAGE_VERSION;
 }
@@ -336,8 +338,9 @@ gda_capi_provider_get_version (GdaServerProvider *provider)
  */
 static gboolean
 gda_capi_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-				   GdaQuarkList *params, GdaQuarkList *auth,
-				   guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data)
+				   GdaQuarkList *params, G_GNUC_UNUSED GdaQuarkList *auth,
+				   G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb,
+				   G_GNUC_UNUSED gpointer cb_data)
 {
 	g_return_val_if_fail (GDA_IS_CAPI_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -459,7 +462,7 @@ gda_capi_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
  */
 static gboolean
 gda_capi_provider_supports_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				      GdaServerOperationType type, GdaSet *options)
+				      GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options)
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -496,7 +499,8 @@ gda_capi_provider_supports_operation (GdaServerProvider *provider, GdaConnection
  */
 static GdaServerOperation *
 gda_capi_provider_create_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				    GdaServerOperationType type, GdaSet *options, GError **error)
+				    GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options,
+				    GError **error)
 {
         gchar *file;
         GdaServerOperation *op;
@@ -596,8 +600,9 @@ gda_capi_provider_render_operation (GdaServerProvider *provider, GdaConnection *
  */
 static gboolean
 gda_capi_provider_perform_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				     GdaServerOperation *op, guint *task_id, 
-				     GdaServerProviderAsyncCallback async_cb, gpointer cb_data, GError **error)
+				     GdaServerOperation *op, G_GNUC_UNUSED guint *task_id,
+				     GdaServerProviderAsyncCallback async_cb, G_GNUC_UNUSED gpointer cb_data,
+				     GError **error)
 {
         GdaServerOperationType optype;
 
@@ -627,8 +632,9 @@ gda_capi_provider_perform_operation (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_begin_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-				     const gchar *name, GdaTransactionIsolation level,
-				     GError **error)
+				     G_GNUC_UNUSED const gchar *name,
+				     G_GNUC_UNUSED GdaTransactionIsolation level,
+				     G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -649,7 +655,7 @@ gda_capi_provider_begin_transaction (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_commit_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-				      const gchar *name, GError **error)
+				      G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -670,7 +676,7 @@ gda_capi_provider_commit_transaction (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_rollback_transaction (GdaServerProvider *provider, GdaConnection *cnc,
-					const gchar *name, GError **error)
+					G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -691,7 +697,7 @@ gda_capi_provider_rollback_transaction (GdaServerProvider *provider, GdaConnecti
  */
 static gboolean
 gda_capi_provider_add_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				 const gchar *name, GError **error)
+				 G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -712,7 +718,7 @@ gda_capi_provider_add_savepoint (GdaServerProvider *provider, GdaConnection *cnc
  */
 static gboolean
 gda_capi_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				      const gchar *name, GError **error)
+				      G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -733,7 +739,7 @@ gda_capi_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_capi_provider_delete_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				    const gchar *name, GError **error)
+				    G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -880,7 +886,7 @@ gda_capi_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnect
  * by the database. See the PostgreSQL provider implementation for an example.
  */
 static GdaSqlParser *
-gda_capi_provider_create_parser (GdaServerProvider *provider, GdaConnection *cnc)
+gda_capi_provider_create_parser (G_GNUC_UNUSED GdaServerProvider *provider, G_GNUC_UNUSED GdaConnection *cnc)
 {
 	TO_IMPLEMENT;
 	return NULL;
@@ -1093,7 +1099,13 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			}
 		}
 		if (!h) {
-			if (! allow_noparam) {
+			if (allow_noparam) {
+                                /* bind param to NULL */
+                                TO_IMPLEMENT;
+                                empty_rs = TRUE;
+                                continue;
+			}
+			else {
 				gchar *str;
 				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
 				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
@@ -1102,17 +1114,17 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
 				g_free (str);
 				break;
-			}
-			else {
-                                /* bind param to NULL */
-                                TO_IMPLEMENT;
-                                empty_rs = TRUE;
-                                continue;
                         }
 
 		}
 		if (!gda_holder_is_valid (h)) {
-			if (! allow_noparam) {
+			if (allow_noparam) {
+                                /* bind param to NULL */
+				TO_IMPLEMENT;
+                                empty_rs = TRUE;
+                                continue;
+			}
+			else {
 				gchar *str;
 				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
 				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
@@ -1121,12 +1133,6 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
 				g_free (str);
 				break;
-			}
-			else {
-                                /* bind param to NULL */
-				TO_IMPLEMENT;
-                                empty_rs = TRUE;
-                                continue;
                         }
 		}
 		else if (gda_holder_value_is_default (h) && !gda_holder_get_value (h)) {
@@ -1165,7 +1171,37 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 
 		/* actual binding using the C API, for parameter at position @i */
 		const GValue *value = gda_holder_get_value (h);
-		TO_IMPLEMENT;
+		if (!value || gda_value_is_null (value)) {
+			GdaStatement *rstmt;
+			if (! gda_rewrite_statement_for_null_parameters (stmt, params, &rstmt, error)) {
+				TO_IMPLEMENT; /* bind to NULL */
+			}
+			else if (!rstmt)
+				return NULL;
+			else {
+				GObject *obj;
+				obj = gda_capi_provider_statement_execute (provider, cnc,
+									   rstmt, params,
+									   model_usage,
+									   col_types,
+									   last_inserted_row,
+									   task_id, async_cb,
+									   cb_data, error);
+				g_object_unref (rstmt);
+				if (GDA_IS_DATA_SELECT (obj)) {
+					GdaPStmt *pstmt;
+					g_object_get (obj, "prepared-stmt", &pstmt, NULL);
+					if (pstmt) {
+						gda_pstmt_set_gda_statement (pstmt, stmt);
+						g_object_unref (pstmt);
+					}
+				}
+				return obj;
+			}
+		}
+		else {
+			TO_IMPLEMENT;
+		}
 	}
 		
 	if (event) {
@@ -1260,7 +1296,7 @@ gda_capi_statement_rewrite (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_start (GdaServerProvider *provider, GdaConnection *cnc, 
-				const GdaXaTransactionId *xid, GError **error)
+				const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1282,7 +1318,7 @@ gda_capi_provider_xa_start (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_end (GdaServerProvider *provider, GdaConnection *cnc, 
-			      const GdaXaTransactionId *xid, GError **error)
+			      const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1303,7 +1339,7 @@ gda_capi_provider_xa_end (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_prepare (GdaServerProvider *provider, GdaConnection *cnc, 
-				  const GdaXaTransactionId *xid, GError **error)
+				  const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1325,7 +1361,7 @@ gda_capi_provider_xa_prepare (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_commit (GdaServerProvider *provider, GdaConnection *cnc, 
-				 const GdaXaTransactionId *xid, GError **error)
+				 const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1346,7 +1382,7 @@ gda_capi_provider_xa_commit (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static gboolean
 gda_capi_provider_xa_rollback (GdaServerProvider *provider, GdaConnection *cnc, 
-				   const GdaXaTransactionId *xid, GError **error)
+				   const GdaXaTransactionId *xid, G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
@@ -1369,7 +1405,7 @@ gda_capi_provider_xa_rollback (GdaServerProvider *provider, GdaConnection *cnc,
  */
 static GList *
 gda_capi_provider_xa_recover (GdaServerProvider *provider, GdaConnection *cnc,
-				  GError **error)
+				  G_GNUC_UNUSED GError **error)
 {
 	CapiConnectionData *cdata;
 
