@@ -1,23 +1,21 @@
-/* GDA provider
- * Copyright (C) 2009 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -228,7 +226,7 @@ gda_web_provider_class_init (GdaWebProviderClass *klass)
 }
 
 static void
-gda_web_provider_init (GdaWebProvider *web_prv, GdaWebProviderClass *klass)
+gda_web_provider_init (G_GNUC_UNUSED GdaWebProvider *web_prv, G_GNUC_UNUSED GdaWebProviderClass *klass)
 {
 }
 
@@ -247,7 +245,8 @@ gda_web_provider_get_type (void)
 			NULL, NULL,
 			sizeof (GdaWebProvider),
 			0,
-			(GInstanceInitFunc) gda_web_provider_init
+			(GInstanceInitFunc) gda_web_provider_init,
+			0
 		};
 		g_static_mutex_lock (&registering);
 		if (type == 0)
@@ -263,7 +262,7 @@ gda_web_provider_get_type (void)
  * Get provider name request
  */
 static const gchar *
-gda_web_provider_get_name (GdaServerProvider *provider)
+gda_web_provider_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return WEB_PROVIDER_NAME;
 }
@@ -272,7 +271,7 @@ gda_web_provider_get_name (GdaServerProvider *provider)
  * Get provider's version, no need to change this
  */
 static const gchar *
-gda_web_provider_get_version (GdaServerProvider *provider)
+gda_web_provider_get_version (G_GNUC_UNUSED GdaServerProvider *provider)
 {
 	return PACKAGE_VERSION;
 }
@@ -331,7 +330,8 @@ do_server_setup (GdaConnection *cnc, WebConnectionData *cdata)
 static gboolean
 gda_web_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
 				  GdaQuarkList *params, GdaQuarkList *auth,
-				  guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data)
+				  G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb,
+				  G_GNUC_UNUSED gpointer cb_data)
 {
 	g_return_val_if_fail (GDA_IS_WEB_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -578,7 +578,7 @@ gda_web_provider_get_server_version (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_web_provider_supports_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				     GdaServerOperationType type, GdaSet *options)
+				     GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options)
 {
 	if (cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -613,7 +613,8 @@ gda_web_provider_supports_operation (GdaServerProvider *provider, GdaConnection
  */
 static GdaServerOperation *
 gda_web_provider_create_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				   GdaServerOperationType type, GdaSet *options, GError **error)
+				   G_GNUC_UNUSED GdaServerOperationType type, G_GNUC_UNUSED GdaSet *options,
+				   GError **error)
 {
 	WebConnectionData *cdata = NULL;
 
@@ -638,7 +639,7 @@ gda_web_provider_create_operation (GdaServerProvider *provider, GdaConnection *c
  */
 static gchar *
 gda_web_provider_render_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				   GdaServerOperation *op, GError **error)
+				   G_GNUC_UNUSED GdaServerOperation *op, GError **error)
 {
 	WebConnectionData *cdata = NULL;
 
@@ -663,8 +664,9 @@ gda_web_provider_render_operation (GdaServerProvider *provider, GdaConnection *c
  */
 static gboolean
 gda_web_provider_perform_operation (GdaServerProvider *provider, GdaConnection *cnc,
-				    GdaServerOperation *op, guint *task_id, 
-				    GdaServerProviderAsyncCallback async_cb, gpointer cb_data, GError **error)
+				    GdaServerOperation *op, G_GNUC_UNUSED guint *task_id,
+				    GdaServerProviderAsyncCallback async_cb, G_GNUC_UNUSED gpointer cb_data,
+				    GError **error)
 {
         GdaServerOperationType optype;
 
@@ -718,7 +720,7 @@ gda_web_provider_begin_transaction (GdaServerProvider *provider, GdaConnection *
 
 	/* prepare XML command */
 	xmlDocPtr doc;
-	xmlNodePtr root, cmdnode;
+	xmlNodePtr root;
 	gchar *token;
 	doc = xmlNewDoc (BAD_CAST "1.0");
 	root = xmlNewNode (NULL, BAD_CAST "request");
@@ -726,7 +728,7 @@ gda_web_provider_begin_transaction (GdaServerProvider *provider, GdaConnection *
 	token = _gda_web_compute_token (cdata);
 	xmlNewChild (root, NULL, BAD_CAST "token", BAD_CAST token);
 	g_free (token);
-	cmdnode = xmlNewChild (root, NULL, BAD_CAST "cmd", BAD_CAST "BEGIN");
+	xmlNewChild (root, NULL, BAD_CAST "cmd", BAD_CAST "BEGIN");
 
 	/* send command */
 	xmlChar *cmde;
@@ -778,7 +780,7 @@ gda_web_provider_commit_transaction (GdaServerProvider *provider, GdaConnection
 
 	/* prepare XML command */
 	xmlDocPtr doc;
-	xmlNodePtr root, cmdnode;
+	xmlNodePtr root;
 	gchar *token;
 	doc = xmlNewDoc (BAD_CAST "1.0");
 	root = xmlNewNode (NULL, BAD_CAST "request");
@@ -786,7 +788,7 @@ gda_web_provider_commit_transaction (GdaServerProvider *provider, GdaConnection
 	token = _gda_web_compute_token (cdata);
 	xmlNewChild (root, NULL, BAD_CAST "token", BAD_CAST token);
 	g_free (token);
-	cmdnode = xmlNewChild (root, NULL, BAD_CAST "cmd", BAD_CAST "COMMIT");
+	xmlNewChild (root, NULL, BAD_CAST "cmd", BAD_CAST "COMMIT");
 
 	/* send command */
 	xmlChar *cmde;
@@ -838,7 +840,7 @@ gda_web_provider_rollback_transaction (GdaServerProvider *provider, GdaConnectio
 
 	/* prepare XML command */
 	xmlDocPtr doc;
-	xmlNodePtr root, cmdnode;
+	xmlNodePtr root;
 	gchar *token;
 	doc = xmlNewDoc (BAD_CAST "1.0");
 	root = xmlNewNode (NULL, BAD_CAST "request");
@@ -846,7 +848,7 @@ gda_web_provider_rollback_transaction (GdaServerProvider *provider, GdaConnectio
 	token = _gda_web_compute_token (cdata);
 	xmlNewChild (root, NULL, BAD_CAST "token", BAD_CAST token);
 	g_free (token);
-	cmdnode = xmlNewChild (root, NULL, BAD_CAST "cmd", BAD_CAST "ROLLBACK");
+	xmlNewChild (root, NULL, BAD_CAST "cmd", BAD_CAST "ROLLBACK");
 
 	/* send command */
 	xmlChar *cmde;
@@ -1002,7 +1004,7 @@ gda_web_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnection
  */
 static gboolean
 gda_web_provider_delete_savepoint (GdaServerProvider *provider, GdaConnection *cnc,
-				   const gchar *name, GError **error)
+				   G_GNUC_UNUSED const gchar *name, G_GNUC_UNUSED GError **error)
 {
 	WebConnectionData *cdata;
 
@@ -1054,7 +1056,7 @@ gda_web_provider_supports_feature (GdaServerProvider *provider, GdaConnection *c
  */
 static GdaDataHandler *
 gda_web_provider_get_data_handler (GdaServerProvider *provider, GdaConnection *cnc,
-				   GType type, const gchar *dbms_type)
+				   G_GNUC_UNUSED GType type, G_GNUC_UNUSED const gchar *dbms_type)
 {
 	WebConnectionData *cdata = NULL;
 
@@ -1077,7 +1079,7 @@ gda_web_provider_get_data_handler (GdaServerProvider *provider, GdaConnection *c
  * This method returns the "preferred" DBMS type for GType
  */
 static const gchar*
-gda_web_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnection *cnc, GType type)
+gda_web_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnection *cnc, G_GNUC_UNUSED GType type)
 {
 	WebConnectionData *cdata = NULL;
 
@@ -1101,7 +1103,7 @@ gda_web_provider_get_default_dbms_type (GdaServerProvider *provider, GdaConnecti
  * by the database. See the PostgreSQL provider implementation for an example.
  */
 static GdaSqlParser *
-gda_web_provider_create_parser (GdaServerProvider *provider, GdaConnection *cnc)
+gda_web_provider_create_parser (G_GNUC_UNUSED GdaServerProvider *provider, GdaConnection *cnc)
 {
 	WebConnectionData *cdata = NULL;
 
@@ -1483,7 +1485,15 @@ gda_web_provider_statement_execute (GdaServerProvider *provider, GdaConnection *
 			}
 		}
 		if (!h) {
-			if (! allow_noparam) {
+			if (allow_noparam) {
+                                /* bind param to NULL */
+				node = xmlNewChild (argsnode, NULL, BAD_CAST "arg", NULL);
+				xmlSetProp (node, BAD_CAST "type", BAD_CAST "NULL");
+                                empty_rs = TRUE;
+                                continue;
+			}
+			else {
+
 				gchar *str;
 				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
 				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
@@ -1492,18 +1502,16 @@ gda_web_provider_statement_execute (GdaServerProvider *provider, GdaConnection *
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
 				g_free (str);
 				break;
-			}
-			else {
+                        }
+		}
+		if (!gda_holder_is_valid (h)) {
+			if (allow_noparam) {
                                 /* bind param to NULL */
-				node = xmlNewChild (argsnode, NULL, BAD_CAST "arg", NULL);
 				xmlSetProp (node, BAD_CAST "type", BAD_CAST "NULL");
                                 empty_rs = TRUE;
                                 continue;
-                        }
-
-		}
-		if (!gda_holder_is_valid (h)) {
-			if (! allow_noparam) {
+			}
+			else {
 				gchar *str;
 				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
 				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
@@ -1512,12 +1520,6 @@ gda_web_provider_statement_execute (GdaServerProvider *provider, GdaConnection *
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
 				g_free (str);
 				break;
-			}
-			else {
-                                /* bind param to NULL */
-				xmlSetProp (node, BAD_CAST "type", BAD_CAST "NULL");
-                                empty_rs = TRUE;
-                                continue;
                         }
 		}
 		else if (gda_holder_value_is_default (h) && !gda_holder_get_value (h)) {
@@ -1557,12 +1559,49 @@ gda_web_provider_statement_execute (GdaServerProvider *provider, GdaConnection *
 
 		/* actual binding using the C API, for parameter at position @i */
 		const GValue *value = gda_holder_get_value (h);
-		gchar *tmp;
-		tmp = gda_value_stringify (value);
-		node = xmlNewTextChild (argsnode, NULL, BAD_CAST "arg", BAD_CAST tmp);
-		g_free (tmp);
-		xmlSetProp (node, BAD_CAST "type",
-			    BAD_CAST gtype_to_webtype (gda_holder_get_g_type (h)));
+		if (!value || gda_value_is_null (value)) {
+			GdaStatement *rstmt;
+			if (! gda_rewrite_statement_for_null_parameters (stmt, params, &rstmt, error)) {
+				gchar *tmp;
+				tmp = gda_value_stringify (value);
+				node = xmlNewTextChild (argsnode, NULL, BAD_CAST "arg", BAD_CAST tmp);
+				g_free (tmp);
+				xmlSetProp (node, BAD_CAST "type",
+					    BAD_CAST gtype_to_webtype (gda_holder_get_g_type (h)));
+			}
+			else if (!rstmt)
+				return NULL;
+			else {
+				GObject *obj;
+				g_object_unref (ps);
+				xmlFreeDoc (doc);
+				obj = gda_web_provider_statement_execute (provider, cnc,
+									  rstmt, params,
+									  model_usage,
+									  col_types,
+									  last_inserted_row,
+									  task_id, async_cb,
+									  cb_data, error);
+				g_object_unref (rstmt);
+				if (GDA_IS_DATA_SELECT (obj)) {
+					GdaPStmt *pstmt;
+					g_object_get (obj, "prepared-stmt", &pstmt, NULL);
+					if (pstmt) {
+						gda_pstmt_set_gda_statement (pstmt, stmt);
+						g_object_unref (pstmt);
+					}
+				}
+				return obj;
+			}
+		}
+		else {
+			gchar *tmp;
+			tmp = gda_value_stringify (value);
+			node = xmlNewTextChild (argsnode, NULL, BAD_CAST "arg", BAD_CAST tmp);
+			g_free (tmp);
+			xmlSetProp (node, BAD_CAST "type",
+				    BAD_CAST gtype_to_webtype (gda_holder_get_g_type (h)));
+		}
 	}
 		
 	if (event) {
