@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2009 - 2010 The GNOME Foundation
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -38,6 +35,7 @@
 #include <libgda-ui/libgda-ui.h>
 #include "data-source-manager.h"
 #include <gdk/gdkkeysyms.h>
+#include "analyser.h"
 
 #define MAIN_PAGE_EDITORS 0
 #define MAIN_PAGE_DATA 1
@@ -84,6 +82,7 @@ struct _DataConsolePrivate {
 	GtkWidget *add_source_button;
 	GtkWidget *add_source_menu;
 	gpointer   add_source_menu_index; /* used to know if @add_source_menu needs to be rebuilt */
+	GSList    *customized_menu_items_list; /* list of GtkMenuItem in @add_source_menu */
 };
 
 static void data_console_class_init (DataConsoleClass *klass);
@@ -186,6 +185,8 @@ data_console_dispose (GObject *object)
 	if (dconsole->priv) {
 		if (dconsole->priv->add_source_menu)
 			gtk_widget_destroy (dconsole->priv->add_source_menu);
+		if (dconsole->priv->customized_menu_items_list)
+			g_slist_free (dconsole->priv->customized_menu_items_list);
 		if (dconsole->priv->params_form)
 			gtk_widget_destroy (dconsole->priv->params_form);
 		if (dconsole->priv->popup_container)
@@ -382,33 +383,33 @@ data_console_new (BrowserConnection *bcnc)
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 5);
 
-	button = browser_make_small_button (FALSE, _("Reset"), GTK_STOCK_CLEAR,
+	button = browser_make_small_button (FALSE, FALSE, _("Reset"), GTK_STOCK_CLEAR,
 					    _("Reset the editor's\ncontents"));
 	dconsole->priv->clear_xml_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (editor_clear_clicked_cb), dconsole);
 
-	button = browser_make_small_button (FALSE, _("Add"), GTK_STOCK_ADD,
+	button = browser_make_small_button (FALSE, TRUE, _("Add"), GTK_STOCK_ADD,
 					    _("Add a new data source"));
 	dconsole->priv->add_source_button = button;
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (add_source_clicked_cb), dconsole);
 
-	button = browser_make_small_button (TRUE, _("Variables"), NULL, _("Show variables needed"));
+	button = browser_make_small_button (TRUE, FALSE, _("Variables"), NULL, _("Show variables needed"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	dconsole->priv->params_toggle = GTK_TOGGLE_BUTTON (button);
 	g_signal_connect (button, "toggled",
 			  G_CALLBACK (variables_clicked_cb), dconsole);
 
-	button = browser_make_small_button (FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute specified\n"
+	button = browser_make_small_button (FALSE, FALSE, _("Execute"), GTK_STOCK_EXECUTE, _("Execute specified\n"
 										      "data manager"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (execute_clicked_cb), dconsole);
 
-	button = browser_make_small_button (TRUE, _("View XML"), NULL, _("View specifications\n"
+	button = browser_make_small_button (TRUE, FALSE, _("View XML"), NULL, _("View specifications\n"
 									 "as XML (advanced)"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	spec_editor_toggled_cb (GTK_TOGGLE_BUTTON (button), dconsole);
@@ -416,7 +417,7 @@ data_console_new (BrowserConnection *bcnc)
 			  G_CALLBACK (spec_editor_toggled_cb), dconsole);
 
 #ifdef HAVE_GDU
-	button = browser_make_small_button (FALSE, _("Help"), GTK_STOCK_HELP, _("Help"));
+	button = browser_make_small_button (FALSE, FALSE, _("Help"), GTK_STOCK_HELP, _("Help"));
 	gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (help_clicked_cb), dconsole);
@@ -749,7 +750,18 @@ static void
 add_source_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 {
 	GdaMetaStruct *mstruct;
+	DataSource *current_source;
+
 	mstruct = browser_connection_get_meta_struct (dconsole->priv->bcnc);
+	current_source = ui_spec_editor_get_selected_source (UI_SPEC_EDITOR (dconsole->priv->ui_sped));
+
+	/* remove customization, if any */
+	if (dconsole->priv->customized_menu_items_list) {
+		g_slist_foreach (dconsole->priv->customized_menu_items_list,
+				 (GFunc) gtk_widget_destroy, NULL);
+		g_slist_free (dconsole->priv->customized_menu_items_list);
+		dconsole->priv->customized_menu_items_list = NULL;
+	}
 
 	if (dconsole->priv->add_source_menu &&
 	    (dconsole->priv->add_source_menu_index != (gpointer) mstruct)) {
@@ -758,15 +770,20 @@ add_source_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 		dconsole->priv->add_source_menu_index = NULL;
 	}
 	if (dconsole->priv->add_source_menu) {
+		if (current_source)
+			dconsole->priv->customized_menu_items_list =
+				data_manager_add_proposals_to_menu (dconsole->priv->add_source_menu,
+								    dconsole->priv->mgr,
+								    current_source, GTK_WIDGET (dconsole));
 		gtk_menu_popup (GTK_MENU (dconsole->priv->add_source_menu), NULL, NULL,
-			NULL, NULL, 0,
-			gtk_get_current_event_time ());
+				NULL, NULL, 0,
+				gtk_get_current_event_time ());
 		return;
 	}
 
 	GtkWidget *menu, *mitem, *submenu = NULL;
 	menu = gtk_menu_new ();
-	mitem = gtk_menu_item_new_with_label (_("Data source from query"));
+	mitem = gtk_menu_item_new_with_label (_("Data source from SQL"));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (add_source_mitem_activated_cb), dconsole);
 	gtk_widget_show (mitem);
@@ -804,7 +821,7 @@ add_source_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 					schema_changed = FALSE;
 
 				if (schema_changed) {
-					str = g_strdup_printf (_("In schema %s"), current_schema);
+					str = g_strdup_printf (_("In schema '%s'"), current_schema);
 					mitem = gtk_menu_item_new_with_label (str);
 					gtk_widget_show (mitem);
 					g_free (str);
@@ -828,6 +845,13 @@ add_source_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataConsole *dconsole)
 	}
 	dconsole->priv->add_source_menu_index = (gpointer) mstruct;
 	dconsole->priv->add_source_menu = menu;
+
+	if (current_source)
+		dconsole->priv->customized_menu_items_list =
+			data_manager_add_proposals_to_menu (dconsole->priv->add_source_menu,
+							    dconsole->priv->mgr,
+							    current_source, GTK_WIDGET (dconsole));
+
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			NULL, NULL, 0,
 			gtk_get_current_event_time ());
@@ -921,7 +945,8 @@ data_console_page_get_actions_group (BrowserPage *page)
 	DataConsole *dconsole;
 	dconsole = DATA_CONSOLE (page);
 	if (! dconsole->priv->agroup) {
-		dconsole->priv->agroup = gtk_action_group_new ("QueryExecConsoleActions");
+		dconsole->priv->agroup = gtk_action_group_new ("DataManagerConsoleActions");
+		gtk_action_group_set_translation_domain (dconsole->priv->agroup, GETTEXT_PACKAGE);
 		gtk_action_group_add_toggle_actions (dconsole->priv->agroup,
 						     ui_actions, G_N_ELEMENTS (ui_actions), page);
 	}
@@ -970,6 +995,26 @@ data_console_get_text (DataConsole *console)
 	return xml_spec_editor_get_xml_text (XML_SPEC_EDITOR (console->priv->xml_sped));
 }
 
+/**
+ * data_console_is_unused
+ */
+gboolean
+data_console_is_unused (DataConsole *console)
+{
+	gchar *text;
+	gboolean retval = TRUE;
+	g_return_val_if_fail (IS_DATA_CONSOLE (console), FALSE);
+	text = data_console_get_text (console);
+	if (!text || !*text) {
+		g_free (text);
+		return TRUE;
+	}
+	if (strcmp (text, DEFAULT_XML))
+		retval = FALSE;
+	g_free (text);
+	return retval;
+}
+
 /**
  * data_console_execute
  * @console: a #DataConsole widget
