@@ -1,23 +1,19 @@
 /*
- * Copyright (C) 2011 The GNOME Foundation
+ * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This Program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -232,9 +228,15 @@ source_drag_data_get_cb (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED GdkDragC
 	case TARGET_KEY_VALUE: {
 		gchar *str;
 		str = ldap_entries_page_to_selection (ebrowser);
+#if GTK_CHECK_VERSION(2,18,0)
 		gtk_selection_data_set (selection_data,
 					gtk_selection_data_get_target (selection_data), 8, (guchar*) str,
 					strlen (str));
+#else
+		gtk_selection_data_set (selection_data,
+					(GdkAtom) selection_data->data, 8, (guchar*) str,
+					strlen (str));
+#endif
 		g_free (str);
 		break;
 	}
@@ -335,13 +337,13 @@ selection_changed_cb (GtkTreeSelection *sel, LdapEntriesPage *ebrowser)
 }
 
 static void
-open_dn_requested_cb (EntryProperties *eprop, const gchar *dn, LdapEntriesPage *ebrowser)
+open_dn_requested_cb (G_GNUC_UNUSED EntryProperties *eprop, const gchar *dn, LdapEntriesPage *ebrowser)
 {
 	ldap_entries_page_set_current_dn (ebrowser, dn);
 }
 
 static void
-open_class_requested_cb (EntryProperties *eprop, const gchar *classname, LdapEntriesPage *ebrowser)
+open_class_requested_cb (G_GNUC_UNUSED EntryProperties *eprop, const gchar *classname, LdapEntriesPage *ebrowser)
 {
 	BrowserPerspective *bpers;
 	bpers = browser_page_get_perspective (BROWSER_PAGE (ebrowser));
