@@ -1,13 +1,13 @@
-/* 
- * Copyright (C) 2007 - 2010 The GNOME Foundation.
+/*
+ * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #undef GDA_DISABLE_DEPRECATED
@@ -28,6 +28,7 @@
 #include <readline/history.h>
 #endif
 #include <sql-parser/gda-statement-struct-util.h>
+#include "tools-utils.h"
 
 /*
  *  gda_internal_command_arg_remove_quotes
@@ -617,17 +618,23 @@ gda_internal_command_detail (G_GNUC_UNUSED SqlConsole *console, GdaConnection *c
 			"ORDER BY table_schema, table_name";
 		model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, NULL);
 
-		/* if no row, then return all the objects from all the schemas */
-		if (model && (gda_data_model_get_n_rows (model) == 0)) {
-			g_object_unref (model);
-			sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
-			"table_owner as Owner FROM _tables "
-			"ORDER BY table_schema, table_name";
-			model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, NULL);
+		if (model) {
+			/* if no row, then return all the objects from all the schemas */
+			if (gda_data_model_get_n_rows (model) == 0) {
+				g_object_unref (model);
+				sql = "SELECT table_schema AS Schema, table_name AS Name, table_type as Type, "
+					"table_owner as Owner FROM _tables "
+					"ORDER BY table_schema, table_name";
+				model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), sql, error, NULL);
+			}
+			res = g_new0 (GdaInternalCommandResult, 1);
+			res->type = GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL;
+			res->u.model = model;
+		}
+		else {
+			res = g_new0 (GdaInternalCommandResult, 1);
+			res->type = GDA_INTERNAL_COMMAND_RESULT_EMPTY;
 		}
-		res = g_new0 (GdaInternalCommandResult, 1);
-		res->type = GDA_INTERNAL_COMMAND_RESULT_DATA_MODEL;
-		res->u.model = model;
 		return res;
 	}
 
@@ -789,6 +796,7 @@ gda_internal_command_detail (G_GNUC_UNUSED SqlConsole *console, GdaConnection *c
 			const gchar *sql = "SELECT constraint_type, constraint_name  "
 				"FROM _table_constraints WHERE table_catalog = ##tc::string "
 				"AND table_schema = ##ts::string AND table_name = ##tname::string "
+				"AND constraint_type != 'FOREIGN KEY' "
 				"ORDER BY constraint_type DESC, constraint_name";
 
 			g_value_set_string ((catalog = gda_value_new (G_TYPE_STRING)), dbo->obj_catalog);
@@ -853,84 +861,7 @@ gda_internal_command_detail (G_GNUC_UNUSED SqlConsole *console, GdaConnection *c
 					}
 				}
 				else if (*str == 'F') {
-					/* foreign key */
-					GdaDataModel *cols;
-					cvalue = gda_data_model_get_value_at (model, 1, i, error);
-					if (!cvalue) {
-						gda_internal_command_exec_result_free (res);
-						res = NULL;
-						goto out;
-					}
-					string = g_string_new (_("Foreign key"));
-					g_string_append_printf (string, " '%s'", g_value_get_string (cvalue));
-
-					str = "SELECT ref_table_schema, ref_table_name, fk_column, ref_column "
-						"FROM _detailed_fk WHERE fk_table_catalog = ##tc::string "
-						"AND fk_table_schema = ##ts::string AND fk_table_name = ##tname::string AND "
-						"fk_constraint_name = ##cname::string "
-						"ORDER BY ordinal_position";
-					
-					cols = gda_meta_store_extract (gda_connection_get_meta_store (cnc), str, NULL, 
-								       "tc", catalog, "ts", schema, "tname", name, "cname", cvalue, 
-								       NULL);
-					if (cols) {
-						gint j, cnrows;
-						cnrows = gda_data_model_get_n_rows (cols);
-						if (cnrows > 0) {
-							g_string_append (string, " (");
-							for (j = 0; j < cnrows; j++) {
-								if (j > 0)
-									g_string_append (string, ", ");
-								cvalue = gda_data_model_get_value_at (cols, 2, j, error);
-								if (!cvalue) {
-									gda_internal_command_exec_result_free (res);
-									res = NULL;
-									g_object_unref (cols);
-									g_string_free (string, TRUE);
-									goto out;
-								}
-								g_string_append (string, g_value_get_string (cvalue));
-							}
-							g_string_append (string, ") references ");
-
-							const GValue *v1, *v2;
-							
-							v1 = gda_data_model_get_value_at (cols, 0, 0, error);
-							if (!v1) {
-								gda_internal_command_exec_result_free (res);
-								res = NULL;
-								g_object_unref (cols);
-								g_string_free (string, TRUE);
-								goto out;
-							}
-							v2 = gda_data_model_get_value_at (cols, 1, 0, error);
-							if (!v2) {
-								gda_internal_command_exec_result_free (res);
-								res = NULL;
-								g_object_unref (cols);
-								g_string_free (string, TRUE);
-								goto out;
-							}
-							g_string_append_printf (string, " %s.%s (", 
-										g_value_get_string (v1),
-										g_value_get_string (v2));
-							for (j = 0; j < cnrows; j++) {
-								if (j > 0)
-									g_string_append (string, ", ");
-								cvalue = gda_data_model_get_value_at (cols, 3, j, error);
-								if (!cvalue) {
-									gda_internal_command_exec_result_free (res);
-									res = NULL;
-									g_object_unref (cols);
-									g_string_free (string, TRUE);
-									goto out;
-								}
-								g_string_append (string, g_value_get_string (cvalue));
-							}
-							g_string_append_c (string, ')');
-						}
-						g_object_unref (cols);
-					}
+					/* foreign key, not handled here */
 				}
 				else if (*str == 'U') {
 					/* Unique constraint */
@@ -989,6 +920,62 @@ gda_internal_command_detail (G_GNUC_UNUSED SqlConsole *console, GdaConnection *c
 			gda_value_free (name);
 
 			g_object_unref (model);
+
+			/* foreign key constraints */
+			GSList *list;
+			for (list = mt->fk_list; list; list = list->next) {
+				GString *string;
+				GdaMetaTableForeignKey *fk;
+				gint i;
+				fk = (GdaMetaTableForeignKey*) list->data;
+				if (GDA_META_TABLE_FOREIGN_KEY_IS_DECLARED (fk))
+					string = g_string_new (_("Declared foreign key"));
+				else
+					string = g_string_new (_("Foreign key"));
+				g_string_append_printf (string, " '%s' (", fk->fk_name);
+				for (i = 0; i < fk->cols_nb; i++) {
+					if (i != 0)
+						g_string_append (string, ", ");
+					g_string_append (string, fk->fk_names_array [i]);
+				}
+				g_string_append (string, ") ");
+				if (fk->depend_on->obj_short_name)
+					/* To translators: the term "references" is the verb
+					 * "to reference" in the context of foreign keys where
+					 * "table A REFERENCES table B" */
+					g_string_append_printf (string, _("references %s"),
+								fk->depend_on->obj_short_name);
+				else
+					/* To translators: the term "references" is the verb
+					 * "to reference" in the context of foreign keys where
+					 * "table A REFERENCES table B" */
+					g_string_append_printf (string, _("references %s.%s"),
+								fk->depend_on->obj_schema,
+								fk->depend_on->obj_name);
+				g_string_append (string, " (");
+				for (i = 0; i < fk->cols_nb; i++) {
+					if (i != 0)
+						g_string_append (string, ", ");
+					g_string_append (string, fk->ref_pk_names_array [i]);
+				}
+				g_string_append (string, ")");
+
+				g_string_append (string, "\n  ");
+				g_string_append (string, _("Policy on UPDATE"));
+				g_string_append (string, ": ");
+				g_string_append (string, tools_utils_fk_policy_to_string (GDA_META_TABLE_FOREIGN_KEY_ON_UPDATE_POLICY (fk)));
+				g_string_append (string, "\n  ");
+				g_string_append (string, _("Policy on DELETE"));
+				g_string_append (string, ": ");
+				g_string_append (string, tools_utils_fk_policy_to_string (GDA_META_TABLE_FOREIGN_KEY_ON_DELETE_POLICY (fk)));
+
+				subres = g_new0 (GdaInternalCommandResult, 1);
+				subres->type = GDA_INTERNAL_COMMAND_RESULT_TXT;
+				subres->u.txt = string;
+				res->u.multiple_results = g_slist_append (res->u.multiple_results,
+									  subres);
+			}
+
 			return res;
 		}
 	}
